﻿//////////////////////////////////////////////////////////
//
//  Headless Write Engine Tutorial Application
//
//  VSO Software 2005
//  R1 : 02/05/2005
//
//////////////////////////////////////////////////////////

unit F_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, TeeProcs, TeEngine, Chart, ComCtrls, StdCtrls, Menus,
  Series,
  // Write Engine Units
  burn.legacy_enginebase,
  burn.command_interface,
  Log.interfaces,
  JvBaseDlg, JvBrowseFolder, JvAppStorage, JvAppIniStorage, JvComponentBase,
  JvFormPlacement, VclTee.TeeGDIPlus;

type
  Tfrm_main = class(TForm)
    tab_main: TTabControl;
    cht_speed: TChart;
    sb_status: TStatusBar;
    mm_menu: TMainMenu;
    mi_file: TMenuItem;
    mi_exit: TMenuItem;
    mi_view: TMenuItem;
    BurnSpeed1: TMenuItem;
    mi_help: TMenuItem;
    mi_about: TMenuItem;
    mi_write: TMenuItem;
    mi_writeiso: TMenuItem;
    mi_writedvd: TMenuItem;
    mi_writestream: TMenuItem;
    OpenDialog: TOpenDialog;
    milog1: TMenuItem;
    mo_log: TMemo;
    mi_action: TMenuItem;
    mi_opentray: TMenuItem;
    mi_closetray: TMenuItem;
    GridPanel1: TGridPanel;
    cbb_Writers: TComboBox;
    cbb_writeaffinity: TComboBox;
    cbb_speed: TComboBox;
    pb_progress: TProgressBar;
    bt_cancel: TButton;
    mi_WriteData: TMenuItem;
    BFF: TJvBrowseForFolderDialog;
    Mi_WriteBluRay: TMenuItem;
    N1: TMenuItem;
    SetISOcreationmode1: TMenuItem;
    frm_storage: TJvFormStorage;
    AppStorage: TJvAppIniFileStorage;
    EdLabel: TEdit;
    N2: TMenuItem;
    LogDiscNotif: TMenuItem;
    mi_writeDataFilelist: TMenuItem;
    CreateISOFilesystemonlyBURNINGSDKFSO1: TMenuItem;
    AskEmptymediasize1: TMenuItem;
    MI_BlankMedia: TMenuItem;
    Realtimespeed: TMenuItem;
    RtSpeedTimer: TTimer;
    procedure mi_closetrayClick(Sender: TObject);
    procedure mi_opentrayClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bt_cancelClick(Sender: TObject);
    procedure cbb_speedChange(Sender: TObject);
    procedure cbb_WritersChange(Sender: TObject);
    procedure mi_burnspeedClick(Sender: TObject);
    procedure mi_exitClick(Sender: TObject);
    procedure mi_logClick(Sender: TObject);
    procedure mi_writeisoClick(Sender: TObject);
    procedure mi_writedvdClick(Sender: TObject);
    procedure mi_writestreamClick(Sender: TObject);
    procedure tab_mainChange(Sender: TObject);
    procedure GridPanel1Click(Sender: TObject);
    procedure mo_logChange(Sender: TObject);
    procedure mi_WriteDataClick(Sender: TObject);
    procedure Mi_WriteBluRayClick(Sender: TObject);
    procedure SetISOcreationmode1Click(Sender: TObject);
    procedure frm_storageAfterRestorePlacement(Sender: TObject);
    procedure LogDiscNotifClick(Sender: TObject);
    procedure mi_writeDataFilelistClick(Sender: TObject);
    procedure CreateISOFilesystemonlyBURNINGSDKFSO1Click(Sender: TObject);
    procedure AskEmptymediasize1Click(Sender: TObject);
    procedure MI_BlankMediaClick(Sender: TObject);
    procedure RtSpeedTimerTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FBurnAllowed: Boolean;
    procedure SetBurnAllowed(const Value: Boolean);

    procedure SetBurning(burning: Boolean);
    procedure SetTab(index: integer);
    procedure UpdateLog;
  public
    { Public declarations }
    Be: TBurningEngine;                   // Burning Engine
    ProgressIntf: IVSOBurningSDKProgress; // Progress interface
    Sectors_To_burn: Longword;
    max_spd: integer;
    FScheduleForClose: boolean;

    procedure BeNotify(Sender: TObject; WriteOp: Longword; Progress: Longword); // Notify CallBack
    procedure BeLog(Sender: TObject;
                    Kind: TLog_Kind;
                    Const Text: string;
                    Const Params: Array of const);// Internal Log callback
    procedure OnParseMedia;
    procedure OnErasing;                        // Erasing Notification Call
    procedure OnDiscAccepted(Media_count: longword); // Media Inserted
    procedure OnDiscRejected(Reason: longword);// Media rejected
    procedure OnFinishing(Progress: Longword);  // Finishing Notification Call
    procedure OnFinished;                       // Finished Notification Call
    procedure OnFormating(Progress: Longword);  // Formating Notification Call
    procedure OnWriting(Progress: Longword);    // Writing Notification Call
    procedure OnDeviceCountChange(nb_devices: Longword);  // Finishing Notification Call

    procedure PrepareRealtimeSpeed;

    property BurnAllowed: Boolean read FBurnAllowed write SetBurnAllowed;
  end;

var
  frm_main: Tfrm_main;

implementation

{$R *.dfm}

{ To use the Burning SDK, a key is mandatory.


  * For evaluation:
    Use a valid email address as a key. This will trigger the sending of an email with a confirmation link.
    You will need to receive the email and click on the confirmation link before you can use the SDK.

    To pass the email to the software, pass a pointer to a ANSI or UTF-8 string with the email address.

    Sample code:

    var
      Key: UTF8String;

    key := 'me@example.org';

    if not Be.SendKey(MarshaledAString(key),length(Key)) then
    begin
      // not accepted
    end;

    NOTE: In this mode, internet connection is mandatory in any case. If no connection could be established, the burning will not start.


  * For commercial use:
    You will receive a long text key under the form 'VSO_V4K_longbase64binarydata='
    Pass this key to the SendKey function, with a optional suffix as described below.

    Optional: To avoid being charged for trial versions, you can append to the SDK key the following statements :

    ' *trial*' to indicate that your software is running in trial mode.
    ' *registered*' or no suffix, to indicate that your software is running in license paid mode.



    Sample code (this key is a sample not valid):
    var
      Key: UTF8String

    Key := 'VSO_V4K_0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+/0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+/0123456789abcdefghijklmnopqrstuvwxyzABCDEFG='
    if iAmInTrial then
      key := key + ' *trial*'
    else
      key := key + ' *registered*';
    if not Be.SendKey(MarshaledAString(key),length(Key)) then
    begin
      // Key not accepted
    end;

    It is also possible to send the key in pure binary form. It's 128 bytes of data, plus optional BOOLEAN for the trial status.

    The activation of the SDK key is done with an online request to VSO activation servers.
      * with ' *trial*' suffix, a temporary activation will be done.
      * with no or ' *registered*' suffix, a definitive activation will be done.
    once the definitive activation is done, the SDK will no longer attempt to connect to the Internet.
    If no internet connection is available, the activation attempt will be postponed.
    don't abuse of *trial* suffix, as I'll cause internet connection attempt from time to time, and will start to deny use of the SDK if those attempts fail over a predefined period of time.

}
{ $Message Fatal 'Change the API key below with the one you have received, and remove this message'}
const
  api_key: array[0..132-1] of byte  = (
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00   // <--- for registered
//    $01, $00, $00, $00   // <--- for trial
  );


const
   test_key: pansichar = 'burningsdk@vsosoftware.fr';

// FormCreate
procedure Tfrm_main.FormCreate(Sender: TObject);
var
  SDKInfo: string;

begin
  // Init Burn Engine
  Be := TBurningEngine.Create('',Handle);                       // Create a TBurningEngine
  if not Be.Hwe_Available then
  begin
     ShowMessage('Burning engine DLL not found in current or visible path (vso_hwe.dll)');
     Close;
     halt(0);
  end;



  // Burn Engine Calls
  Be.OnNotify := BeNotify;                                      // Set OnNotify Event
  Be.OnLog    := BeLog;                                         // Set internal logging of Hwe
  Be.SetLogfile(ChangeFileExt(Application.ExeName,'.log'));     // Tell the engine to write log file into app's install folder
  if Be.GetSDKVersionInformation(SDKInfo) then                  // Get SDK version information
    mo_Log.Lines.Add(SDKInfo);




  if not Be.SendKey(test_key,strlen(test_key)) then
//  if not Be.SendKey(@api_key,sizeof(api_key)) then // Send commercial Key to burning engine
  begin
    // something went wrong with the key. Burn will not work in anyway.
     ShowMessage('Key not accepted. Application will now close.');
     FScheduleForClose := True;
     exit;
  end;

  Be.GetWriterList(cbb_Writers.Items);                          // Get list of Writers from burning engine
  // Init User Interface
  if cbb_Writers.Items.Count > 0 then
  begin
    cbb_Writers.ItemIndex := 0;
    cbb_Writers.Enabled := true;
    cbb_writers.Hint := 'Please insert a (re)writable CD, DVD or Blu-Ray';
  end
  else begin
    cbb_Writers.Enabled := false;
    cbb_writers.Hint := 'A (re)writable CD, DVD or Blu-Ray is inserted, Burning can start';
  end;
  cbb_speed.Enabled := false;
  cht_speed.AddSeries(TFastLineSeries.Create(cht_speed));
  cht_speed.Series[0].Title := 'Instant burning speed';
  cht_speed.AddSeries(TFastLineSeries.Create(cht_speed));
  cht_speed.Series[1].Title := 'Average burning speed';
  SetBurning(false);
  // Tell the burning engine to start detection of inserted media.
  Be.EnablePolling := true;
  SetTab(0);
end;

// FormDestroy
procedure Tfrm_main.FormDestroy(Sender: TObject);
begin
  // Close Burn Engine
  // Kill all instances of ProgressIntf first, as it has been allocated in the DLL
  ProgressIntf := nil;
  BE.Free;
end;


procedure Tfrm_main.FormShow(Sender: TObject);
begin
  if FScheduleForClose then
  begin
    Close;
  end;
end;

procedure Tfrm_main.frm_storageAfterRestorePlacement(Sender: TObject);
begin
  cbb_WritersChange(Sender);
end;

procedure Tfrm_main.GridPanel1Click(Sender: TObject);
begin

end;

// bt_cancelClick
procedure Tfrm_main.bt_cancelClick(Sender: TObject);
begin
  // Burn Engine Calls
  Be.AbortQuery;  // Abort the current write operation

  // Init User Interface
  SetBurning(false);
end;


// cbb_WritersChange
procedure Tfrm_main.cbb_WritersChange(Sender: TObject);
begin
  // Burn Engine Calls
  Be.SetWriter(cbb_Writers.ItemIndex); // Set the writing device to use

  // Update User Interface
  cbb_speed.Items.Clear;
end;

procedure Tfrm_main.CreateISOFilesystemonlyBURNINGSDKFSO1Click(Sender: TObject);
begin
  if not DirectoryExists(BFF.Directory) then
    BFF.Directory := GetCurrentDir;
  BFF.Directory := frm_storage.ReadString('iso_storage_folder',BFF.Directory);
  if BFF.Execute then
  begin
    frm_storage.WriteString('iso_storage_folder',Bff.Directory);
    Be.IsoPath := BFF.Directory + '\BURNING_SDK.FSO';
  end;

end;

procedure Tfrm_main.MI_BlankMediaClick(Sender: TObject);
begin
  Be.AsyncBlank(false);
end;

// cbb_speedChange
procedure Tfrm_main.cbb_speedChange(Sender: TObject);
begin
  // Burn Engine Calls
  Be.SetWriteSpeed(cbb_speed.Items,cbb_speed.ItemIndex); // Set speed of current writer
end;

// mi_burnspeedClick
procedure Tfrm_main.mi_burnspeedClick(Sender: TObject);
begin
  // Update User Interface
  SetTab(0);
end;

// mi_closetrayClick
procedure Tfrm_main.mi_closetrayClick(Sender: TObject);
begin
  // Burn Engine Calls
  Be.CloseTray;
end;

// mi_exitClick
procedure Tfrm_main.mi_exitClick(Sender: TObject);
begin
  // Terminate Application
  Application.Terminate;
end;

// mi_logClick
procedure Tfrm_main.mi_logClick(Sender: TObject);
begin
  // Update User Interface
  SetTab(1);
end;

// mi_writedvdClick
procedure Tfrm_main.mi_writeDataFilelistClick(Sender: TObject);
var
  Filelist : TStringlist;
  I : integer;
begin
  // OpenDialog
  OpenDialog.Filter := 'All files|*.*';
  if not DirectoryExists(OpenDialog.InitialDir) then
    OpenDialog.InitialDir := GetCurrentDir;
  OpenDialog.InitialDir := frm_storage.ReadString('data_folder',OpenDialog.InitialDir);

  if OpenDialog.Execute then // Open dialog to select an VIDEO_TS.IFO file
  begin
    frm_storage.WriteString('data_folder',Bff.Directory);
    // Fill filelist
    Filelist := TStringlist.Create;
    Filelist.Add(HWE_PARAM_FILELIST);
    for i := 0 to OpenDialog.Files.Count - 1 do
      if FileExists(OpenDialog.Files[I]) then
        FileList.Add(OpenDialog.Files[I]);
    if Filelist.Count < 2 then
      begin
        ShowMessage('No valid file selected, can''t start to burn');
        Filelist.Free;
        exit;
      end;
    // Burn Engine Calls
    UpdateLog; // Update log
    if cbb_writeaffinity.ItemIndex >= 0 then
      Be.DVDRWritingModeAffinity := cbb_writeaffinity.ItemIndex;
    PrepareRealtimeSpeed;
    Be.AsyncDataStart(FileList.Text,EdLabel.Text);  // Launch asynchronous filelist burn
    Filelist.Free;
    // Update User Interface
    cbb_Writers.Enabled := False;
    cbb_speed.Enabled := False;
    cbb_writeaffinity.Enabled := False;
    SetTab(1);
    SetBurning(true);
//    InitGraph;
  end;
end;

procedure Tfrm_main.mi_writedvdClick(Sender: TObject);
var
  Path: widestring;
begin
  // OpenDialog
  OpenDialog.Filter := 'VIDEO_TS.IFO|VIDEO_TS.IFO';
  if not DirectoryExists(OpenDialog.InitialDir) then
    OpenDialog.InitialDir := GetCurrentDir;
  OpenDialog.InitialDir := frm_storage.ReadString('dvd_folder',OpenDialog.InitialDir);
  if OpenDialog.Execute then // Open dialog to select an VIDEO_TS.IFO file
  begin
    frm_storage.WriteString('dvd_folder',Bff.Directory);
    // Compute Path
    Path := OpenDialog.FileName;
    while Path[length(Path)] <> '\' do // Get Path Of VIDEO_TS.IFO file
      SetLength(Path,length(Path)-1);

    // Burn Engine Calls
    UpdateLog;                    // Update log
    if cbb_writeaffinity.ItemIndex >= 0 then
      Be.DVDRWritingModeAffinity := cbb_writeaffinity.ItemIndex;
    PrepareRealtimeSpeed;
    Be.AsyncStart(Path,EdLabel.Text);  // Launch asynchronous burn

    // Update User Interface
    cbb_Writers.Enabled := False;
    cbb_speed.Enabled := False;
    cbb_writeaffinity.Enabled := False;
    SetTab(1);
    SetBurning(true);
//    InitGraph;
  end;
end;

// mi_writeisoClick
procedure Tfrm_main.mi_writeisoClick(Sender: TObject);
Var
  Sl: TStringList;
begin
  // OpenDialog
  OpenDialog.Filter := 'ISO Files|*.iso|All files|*.*';
  if not DirectoryExists(OpenDialog.InitialDir) then
    OpenDialog.InitialDir := GetCurrentDir;
  OpenDialog.InitialDir := frm_storage.ReadString('iso_folder',OpenDialog.InitialDir);
  if OpenDialog.Execute then // Open dialog to select an VIDEO_TS.IFO file
  begin
    frm_storage.WriteString('iso_folder',OpenDialog.InitialDir);
    // Burn Engine Calls
    UpdateLog;                           // Update log
    Sl := TStringList.Create;
    Sl.Assign(OpenDialog.Files);
    Sl.Sort;
    Sl.QuoteChar := '"';
    Sl.Delimiter := ';';
    if cbb_writeaffinity.ItemIndex >= 0 then
      Be.DVDRWritingModeAffinity := cbb_writeaffinity.ItemIndex;
    PrepareRealTimeSpeed;
    Be.AsyncIsoStart(Sl.DelimitedText);  // Launch asynchronous burn
    Sl.Free;

    // Update User Interface
    cbb_Writers.Enabled := False;
    cbb_speed.Enabled := False;
    cbb_writeaffinity.Enabled := False;
    SetTab(1);
    SetBurning(true);
//    InitGraph;
  end;
end;

// mi_writestreamClick
procedure Tfrm_main.mi_writestreamClick(Sender: TObject);
//var
//  nSect: integer;
begin
//  // Burn Engine Calls
//  nSect := GetNumberOfSectorsOfEmptyMedia(Be.MediaType) and (not $FF);
////  nSect := 4169920;
//  if cbb_writeaffinity.ItemIndex >= 0 then
//    Be.DVDRWritingModeAffinity := cbb_writeaffinity.ItemIndex;
//  Isc := Be.AsyncStreamStart(nil,nSect,true);
//
//  // Update User Interface
//  cbb_Writers.Enabled := False;
//  cbb_speed.Enabled := False;
//  cbb_writeaffinity.Enabled := False;
//  SetTab(0);
//  initGraph(Isc);
end;

procedure Tfrm_main.mo_logChange(Sender: TObject);
begin

end;

// mi_opentrayClick
procedure Tfrm_main.mi_opentrayClick(Sender: TObject);
begin
  // Burn Engine Calls
  Be.OpenTray;
end;

procedure Tfrm_main.Mi_WriteBluRayClick(Sender: TObject);
var
  Path: widestring;
begin
  BFF.Directory := frm_storage.ReadString('bd_folder',BFF.Directory);
  if not DirectoryExists(BFF.Directory) then BFF.Directory := GetCurrentDir;

  if BFF.Execute(Handle) then
  begin
    frm_storage.WriteString('bd_folder',Bff.Directory);
    // Compute Path
    Path := BFF.Directory;

    // Burn Engine Calls
    UpdateLog;                    // Update log
    PrepareRealtimeSpeed;
    Be.AsyncBdvStart(Path,EdLabel.Text);  // Launch asynchronous burn

    // Update User Interface
    cbb_Writers.Enabled := False;
    cbb_speed.Enabled := False;
    cbb_writeaffinity.Enabled := False;
    SetTab(1);
    SetBurning(true);
//    InitGraph;
  end;
end;

// tab_mainChange
procedure Tfrm_main.tab_mainChange(Sender: TObject);
begin
  // Update User Interface
  case tab_main.TabIndex of
    0: begin
      cht_speed.Visible := true;
      mo_log.Visible := false;
    end;
    1: begin
      cht_speed.Visible := false;
      mo_log.Visible := true;
    end;
  end;
end;



// BeNotify
procedure Tfrm_main.BeNotify(Sender: TObject; WriteOp: Longword; Progress: Longword);
begin
  If GetCurrentThreadId <> MainThreadID then
  begin
    OutputDebugString('Received in other thread context');
  end;
  case TWriteOperation(WriteOp) of
    WRO_BROWSE_FILES: begin
    end;
    WRO_COPY_DATA_TO_CACHE: begin
    end;
    WRO_PARSING_DISC: begin
      OnParseMedia;
    end;
    WRO_DISC_ACCEPTED: begin
      OnDiscAccepted(Progress);
    end;
    WRO_DISC_REJECTED: begin
      OnDiscRejected(Progress);
    end;
    WRO_ERASING:
      onErasing;
    WRO_FORMATTING:
      onFormating(Progress);
    WRO_WRITING:
      onWriting(Progress);
    WRO_FINISIHING:
      onFinishing(Progress);
    WRO_FINISHED:
      onFinished;
    WRO_LOG: begin
    end;
    WRO_DEVICE_COUNT_CHANGED: begin
      OnDeviceCountChange(Progress);
    end;
    else begin
    end;
  end;
end;

// OnErasing
procedure Tfrm_main.OnErasing;  // Erasing Notification Call
begin
  // Update User Interface
  sb_status.Panels.Items[0].Text := 'Erasing ...';
end;

// OnFinishing
procedure Tfrm_main.OnFinishing(Progress: Longword);  // Finishing Notification Call
begin
  // Update User Interface
  sb_status.Panels.Items[0].Text := 'Finishing ...';
  pb_progress.Position := Progress;
end;

// OnFinished
procedure Tfrm_main.OnFinished;  // Finished Notification Call
var
  CurrentWriteStatus: integer;
begin
  // Burn Engine Calls
  CurrentWriteStatus := Be.WriteStatus;                     // Get the current write operation
  mo_Log.Lines.Add('');
  case CurrentWriteStatus of
    HWS_INITOK:                 // Write init ok - write process has started
      mo_Log.Lines.Add('Write Status: Write init ok - write process has started');
    HWS_NOFILES:                // Passed folder contained no files
      mo_Log.Lines.Add('Write Status: Passed folder contained no files');
    HWS_MISSINGIFOORBUP:        // IFO / BUP are missing
      mo_Log.Lines.Add('Write Status: IFO / BUP are missing');
    HWS_BADIFO:                 // IFO contains invalid data. Will not work on some dvd players
      mo_Log.Lines.Add('Write Status: IFO contains invalid data. Will not work on some dvd players');
    HWS_CANTCHECKIFO:           // Access to files can't be gained
      mo_Log.Lines.Add('Write Status: Access to files can''t be gained');
    HWS_TOOBIG:                 // Files are too big to fit in the currently inserted disc
      mo_Log.Lines.Add('Write Status: Files are too big to fit in the currently inserted disc');
    HWS_BADMEDIASTATUS:         // Media is not ready to be written (see above)
      mo_Log.Lines.Add('Write Status: Media is not ready to be written (see above)');
    HWS_BLANKMEDIAFAILED:       // Erasing a RW media has failed
      mo_Log.Lines.Add('Write Status: Erasing a RW media has failed');
    HWS_NOMORESPACEFORTEMPFILE: // There is not enough space in temp folder to setup the temp file
      mo_Log.Lines.Add('Write Status: There is not enough space in temp folder to setup the temp file');
    HWS_CANTBUILDIMAGE:         // Image can''t be built - lot of cause, including impossibility to create a temp file. in temp folder
      mo_Log.Lines.Add('Write Status: Image can''t be built - lot of cause, including impossibility to create a temp file. in temp folder');
    HWS_FORMATFAILED:           // Formatting media has failed
      mo_Log.Lines.Add('Write Status: Formatting media has failed');
    HWS_DEVICEBUSY:             // Device is busy
      mo_Log.Lines.Add('Write Status: Device is busy');
    HWS_DEVICEINITFAILED:       // initialization of the writing device has failed
      mo_Log.Lines.Add('Write Status: initialization of the writing device has failed');
    HWS_STARTBUFFERFAILED:      // Failed to start the data buffering. Probably a windows out of memory
      mo_Log.Lines.Add('Write Status: Failed to start the data buffering. Probably a windows out of memory');
    HWS_BADMEDIA:               // Media is bad
      mo_Log.Lines.Add('Write Status: Media is bad');
    HWS_FINALIZATIONFAILED:     // Finalization process has failed
      mo_Log.Lines.Add('Write Status: Finalization process has failed');
    HWS_FAILED_WRITING:          // Writing failed (write error or process aborted)
      mo_Log.Lines.Add('Write Status: Writing failed (write error or process aborted)');
    HWS_WRITINGCOMPLETE:        // Write is complete
      mo_Log.Lines.Add('Write Status: Write is complete');
    HWS_KEYCHECK:               // Unlock key check has failed
      mo_Log.Lines.Add('Write Status: Unlock key check has failed');
  end;

  // Update User Interface
  SetBurning(false);
  cbb_Writers.Enabled := true;
  cbb_speed.Enabled := true;
  cbb_writeaffinity.Enabled := true;

  sb_status.Panels.Items[0].Text := 'Finished. Please insert another DVD media ...';
  pb_progress.Position := 0;
end;

// OnFormating
procedure Tfrm_main.OnFormating(Progress: Longword);  // Formating Notification Call
begin
  // Update User Interface
  sb_status.Panels.Items[0].Text := 'Formating ...';
  pb_progress.Position := round(pb_progress.max * Progress / $FFFF);
end;

// OnWriting
procedure Tfrm_main.OnWriting(Progress: Longword);  // Writing Notification Call
begin
  // Update User Interface
  SetBurning(true);
  sb_status.Panels.Items[0].Text := 'Burning ...';
  pb_progress.Position := round(Progress / 655.35);
end;

procedure Tfrm_main.PrepareRealtimeSpeed;
begin
  if Realtimespeed.Checked then
  begin
    if Be.GetProgressInterface(ProgressIntf) then
    begin
      cht_speed.Series[0].Clear;
      RtSpeedTimer.Enabled := True;
    end;
  end else begin
    ProgressIntf := nil;
    RtSpeedTimer.Enabled := True;
  end;
end;

procedure Tfrm_main.RtSpeedTimerTimer(Sender: TObject);
var
  cs: integer;
  Dt: single;
begin
  if not assigned(ProgressIntf) then
  begin
    RtSpeedTimer.Enabled := False;
    exit;
  end;
  Dt := ProgressIntf.GetCurrentDataSpeedKBPS(cs,4);
  if Dt = 0 then exit;
  cht_speed.SeriesList[0].AddXY(Cs,Dt);
  if cht_speed.SeriesList.Count = 2 then
  begin
    Dt := ProgressIntf.GetCurrentDataSpeedKBPS(cs,0);
    cht_speed.SeriesList[1].AddXY(Cs,Dt);
  end;

  if ProgressIntf.GetCurrentOperation in [burn_finishing,burn_success,burn_error] then
  begin
    ProgressIntf := nil;
    RtSpeedTimer.Enabled := False;
  end;
end;

// OnMediaInserted
procedure Tfrm_main.OnDeviceCountChange(nb_devices: Longword);
begin
  mo_log.Lines.Add(format('Number of devices has changed. Now %d',[nb_devices]));
  Be.GetWriterList(cbb_Writers.Items);                          // Get list of Writers from burning engine
end;

procedure Tfrm_main.OnDiscAccepted(Media_count: longword);
var
  I: Integer;  // Media Inserted Notification Call
begin
  if Be.WriteAllowed then
  begin

    if (Be.EmptyMediaSize <= Sectors_To_burn) then
    begin
      // Media is NOT big enough
      sb_status.Panels.Items[0].Text := format('Incorrect media. Use one of these :' + #13#10 + '%s',
        [Be.SuitableMediaList(Sectors_To_burn)]);
      // Open tray
      Be.OpenTray;
      BurnAllowed := false;
    end else begin
      // Media is big enough
      BE.GetMediaWriteSpeeds(cbb_speed.Items); // List of available burn speeds
      // Preselect a speed if required, controlled by max_spd value
      if cbb_speed.Items.Count > 0 then
      begin
        if max_spd <= 0 then
        begin
          cbb_speed.ItemIndex := 0
        end else begin
          I := 0;
          While (I < cbb_speed.Items.Count) and (integer(cbb_speed.Items.Objects[I]) > max_spd) do
          begin
            inc(I);
          end;
          if I < cbb_speed.Items.Count then
            cbb_speed.ItemIndex := I
          else
            cbb_speed.ItemIndex := cbb_speed.Items.Count - 1;
        end;
      end;
      if (Media_count and $8000) <> 0 then
        sb_status.Panels.Items[0].Text := format('%s - %s available. Erase needed.',
         [GetMediaName(Be.MediaType), inttomo(int64(Be.EmptyMediaSize) shl 11)])
      else
        sb_status.Panels.Items[0].Text := format('%s - %s available',
          [GetMediaName(Be.MediaType), inttomo(int64(Be.EmptyMediaSize) shl 11)]);
      BurnAllowed := True;
    end;
  end else begin
      sb_status.Panels.Items[0].Text := format('Media %s not writable, or no media',
      [GetMediaName(Be.MediaType)]);
      BurnAllowed := False;
  end;
  screen.Cursor := crDefault;
end;


procedure Tfrm_main.OnDiscRejected(Reason: longword);
var
  DRR: TDiscRejectionReason;
begin
  Drr := TDiscRejectionReason(Reason);
  case Drr of
    DRR_UNKNOWN: sb_status.Panels.Items[0].Text := 'Disc rejected';
    DRR_DISC_NOT_EMPTY: sb_status.Panels.Items[0].Text := format('Disc %s is not empty. Use a blank writable disc.',[GetMediaName(Be.MediaType)]);
    DRR_DISC_NOT_READABLE: sb_status.Panels.Items[0].Text := 'Disc is not readable. Use another disc.';
    DRR_PARSING_INTERRUPTED_NO_DISC: sb_status.Panels.Items[0].Text := 'Disc ejected before I can determine its type.';
    DRR_PARSING_INTERRUPTED_TRAY_OPENED: sb_status.Panels.Items[0].Text := 'Tray opened before I can determine the disc type';
    DRR_NO_DISC: sb_status.Panels.Items[0].Text := 'No disc present in the drive.';
    DRR_NO_DISC_TRAY_OPENED: sb_status.Panels.Items[0].Text := 'Tray opened. Insert disc and close tray.';
    DRR_INCOMPATIBLE_DISC: sb_status.Panels.Items[0].Text := 'This disc is not compatible with this device';
    DRR_INCOMPATIBLE_WRITE: sb_status.Panels.Items[0].Text := format('This %s type / model / brand cannot be written by this device',[GetMediaName(Be.MediaType)]);
    DRR_HARDWARE_FAILURE: sb_status.Panels.Items[0].Text := 'Poor media quality or Drive failure';
    DRR_WRONG_DISC_TYPE: sb_status.Panels.Items[0].Text := format('Disc %s is not the correct type for the project',[GetMediaName(Be.MediaType)]);
  end;
  BurnAllowed := False;
  Screen.Cursor := crDefault;
end;

procedure Tfrm_main.OnParseMedia;
begin
  screen.Cursor := crHourGlass;
  sb_status.Panels.Items[0].Text := 'Inserted disc is being parsed. Please wait a few second ...';
end;

// InitGraph
//procedure Tfrm_main.InitGraph(i: PHweInputStreamContext = nil);
//begin
//  PrevWritten := 0;
//  isc := i;
//end;

// SetBurnAllowed
procedure Tfrm_main.SetBurnAllowed(const Value: Boolean);
begin
  FBurnAllowed := Value;
  mi_write.Enabled := Value;
  mi_writeiso.Enabled := Value;
  mi_writedvd.Enabled := Value;
  mi_writestream.Enabled := Value;
  mi_WriteData.Enabled := Value;
  mi_writeDataFilelist.Enabled := Value;
  Mi_WriteBluRay.Enabled := Value and (Be.MediaType in [MEDIATYPE_BDRS,MEDIATYPE_BDRR,MEDIATYPE_BDRE]);
  cbb_speed.Enabled := Value;
  MI_BlankMedia.Enabled := Be.BlankAllowed;
end;

// SetBurning
procedure Tfrm_main.SetBurning(burning: Boolean);
begin
  // Update User Interface
  bt_cancel.Enabled := burning;
  mi_file.Enabled := not burning;
  mi_view.Enabled := not burning;
  mi_help.Enabled := not burning;
  mi_write.Enabled := not burning;
  mi_action.Enabled := not burning;
end;

procedure Tfrm_main.SetISOcreationmode1Click(Sender: TObject);
begin
  if not DirectoryExists(BFF.Directory) then
    BFF.Directory := GetCurrentDir;
  BFF.Directory := frm_storage.ReadString('iso_storage_folder',BFF.Directory);
  if BFF.Execute then
  begin
    frm_storage.WriteString('iso_storage_folder',Bff.Directory);
    Be.IsoPath := BFF.Directory + '\BURNING_SDK.ISO';
  end;
end;

// SetTab
procedure Tfrm_main.SetTab(index: integer);
begin
  // Update User Interface
  tab_main.TabIndex := index;
  tab_mainChange(self);
end;


// UpdateLog
procedure Tfrm_main.UpdateLog;
var
  CurrentDosDevice: ansichar;
  CurrentMediaStatus: integer;
  CurrentMediaType: integer;
  CurrentWriter: integer;
  CurrentWriterName: widestring;
  DeviceList: TStringList ;
  EmptyMediaSize: Cardinal;
  MediaVolumeName: widestring;
  MediaWriteSpeedList: TStringList ;
  NumberOfWriters: integer;
  NumberOfReaders: integer;
  WriterList: TStringList ;
  WriteSpeedList: TStringList ;

begin
  // Create Instances
  DeviceList := TStringList.Create;
  MediaWriteSpeedList := TStringList.Create;
  WriterList := TStringList.Create;
  WriteSpeedList := TStringList.Create;

  // Variable Initialisation
  CurrentWriter := self.cbb_Writers.ItemIndex; // Get selected writer index from combobox (cbb_Writers)

  // Burn Engine Calls
  CurrentDosDevice := Be.GetWriterDosDevice(CurrentWriter); // Get the DOS letter of a given drive (writing device)
  CurrentMediaStatus := Be.MediaStatus;                     // Get the media status (write readiness) of the currently selected drive
  CurrentMediaType := Be.MediaType;                         // Get the media type of the inserted media in the selected writer
  CurrentWriterName := Be.WriterName[CurrentWriter];        // Get the writer's friendly name string of a given drive (writing device)
  Be.GetDeviceList(DeviceList);                             // Get the writer's friendly name list of CD/DVD drives
  EmptyMediaSize := Be.EmptyMediaSize;                      // Get the absolute maximum size of the inserted media in the currently selected drive
  MediaVolumeName := Be.MediaVolumeName;                    // Get the media volume name of the inserted media in the currently selected drive
  Be.GetMediaWriteSpeeds(MediaWriteSpeedList);              // Get the media write speeds list of the currently selected drive
  NumberOfWriters := Be.WriterCount;                        // Get the number of WRITERS
  NumberOfReaders := Be.ReaderCount;                        // Get the number of DVD READERS
  Be.GetWriterList(WriterList);                             // Get the writer's friendly name list of WRITERS
  Be.GetWriteSpeeds(WriteSpeedList);                        // Get the write speeds list of the currently selected drive

  // Update User Interface
  mo_log.Lines.Clear;
  mo_Log.Lines.Add(Format('Current Writer %s: %s',[CurrentDosDevice,CurrentWriterName]));

  case CurrentMediaType of
    MEDIATYPE_UNKNOWN:      // No profile
      mo_Log.Lines.Add('Media Type: No profile');
    MEDIATYPE_NRDISC:       // Non removable disc
      mo_Log.Lines.Add('Media Type: Non removable disc');
    MEDIATYPE_RMDISC:       // Removable Disc
      mo_Log.Lines.Add('Media Type: Removable Disc');
    MEDIATYPE_MOE:          // Magneto Optical Erasable
      mo_Log.Lines.Add('Media Type: Magneto Optical Erasable');
    MEDIATYPE_OPTICWO:      // Optical Write Once
      mo_Log.Lines.Add('Media Type: Optical Write Once');
    MEDIATYPE_ASMO:         // Advanced Storage - Magneto Optical
      mo_Log.Lines.Add('Media Type: Advanced Storage - Magneto Optical');
    MEDIATYPE_CDROM:        // CD-ROM (not writable)
      mo_Log.Lines.Add('Media Type: CD-ROM (not writable)');
    MEDIATYPE_CDR:          // CD-R (Write once)
      mo_Log.Lines.Add('Media Type: CD-R (Write once)');
    MEDIATYPE_CDRW:         // CD-RW (R/W)
      mo_Log.Lines.Add('Media Type: CD-RW (R/W)');
    MEDIATYPE_DVDROM:       // DVD-ROM (not writable)
      mo_Log.Lines.Add('Media Type: DVD-ROM (not writable)');
    MEDIATYPE_DVDR:         // DVD-R
      mo_Log.Lines.Add('Media Type: DVD-R');
    MEDIATYPE_DVDRAM:       // DVD-RAM
      mo_Log.Lines.Add('Media Type: DVD-RAM');
    MEDIATYPE_DVDRW:        // DVD-RW Restricted overwrite
      mo_Log.Lines.Add('Media Type: DVD-RW Restricted overwrite');
    MEDIATYPE_DVDRWSEQ:     // DVD-RW Sequential Recording
      mo_Log.Lines.Add('Media Type: DVD-RW Sequential Recording');
    MEDIATYPE_DVDPLUSRW:    // DVD+RW
      mo_Log.Lines.Add('Media Type: DVD+RW');
    MEDIATYPE_DVDPLUSR:     // DVD+R
      mo_Log.Lines.Add('Media Type: DVD+R');
    MEDIATYPE_DDCDROM:      // Double Density CD-ROM
      mo_Log.Lines.Add('Media Type: Double Density CD-ROM');
    MEDIATYPE_DDCDR:        // Double Density CD-R (Write once)
      mo_Log.Lines.Add('Media Type: Double Density CD-R (Write once)');
    MEDIATYPE_DDCDRW:       // Double Density CD-RW
      mo_Log.Lines.Add('Media Type: Double Density CD-RW');
    MEDIATYPE_DLDVDPLUSRW:  // Dual Layer DVD+RW
      mo_Log.Lines.Add('Media Type: Dual Layer DVD+RW');
    MEDIATYPE_DLDVDPLUSR:   // Dual Layer DVD+R
      mo_Log.Lines.Add('Media Type: Dual Layer DVD+R');
    MEDIATYPE_HDBURNCDROM:  // HDBurn CD-R
      mo_Log.Lines.Add('Media Type: HDBurn CD-ROM (Not writable)');
    MEDIATYPE_HDBURNCDR:    // HDBurn CD-R
      mo_Log.Lines.Add('Media Type: HDBurn CD-R');
    MEDIATYPE_HDBURNCDRW:   // HDBurn CD-RW
      mo_Log.Lines.Add('Media Type: HDBurn CD-RW');
    MEDIATYPE_DLDVDR:       // DL DVD-R
      mo_Log.Lines.Add('Media Type: DL DVD-R');
    MEDIATYPE_DLDVDRJMP:    // DL DVD-R, Jump recording
      mo_Log.Lines.Add('Media Type: DL DVD-R, Jump recording');
    MEDIATYPE_BDROM:        // Blu Ray DVD-ROM
      mo_Log.Lines.Add('Media Type: Blu Ray DVD-ROM (not writable)');
    MEDIATYPE_BDRS:          // Blu Ray DVD-R
      mo_Log.Lines.Add('Media Type: Blu Ray DVD-R');
    MEDIATYPE_BDRE:         // Blu Ray DVD-RE
      mo_Log.Lines.Add('Media Type: Blu Ray DVD-RE');
    MEDIATYPE_BDRR:
      mo_Log.Lines.Add('Media Type: Blu Ray DVD-RE');
  end;

  mo_Log.Lines.Add(Format('Media Volume Name: %s',[MediaVolumeName]));
  mo_Log.Lines.Add(Format('Empty Media Size: %d',[EmptyMediaSize]));

  case CurrentMediaStatus of
    HWE_NO_SELECTED_DRIVE:  // No selected writers available
      mo_Log.Lines.Add('Media status : No selected writers available');
    HWE_READY_TO_WRITE:     // Empty DVD (re)writable Media is inserted - Write can start
      mo_Log.Lines.Add('Media status : Empty DVD (re)writable Media is inserted - Write can start');
    HWE_FULL_RW:            // Full rewritable DVD - write can start, but existing data on DVD will be lost
      mo_Log.Lines.Add('Media status : Full rewritable DVD - write can start, but existing data on DVD will be lost');
    HWE_NOT_READY:          // Drive is not ready (no media detected)
      mo_Log.Lines.Add('Media status : Drive is not ready (no media detected)');
    HWE_MEDIA_NOT_WRITABLE: // Detected media is full and can''t be erased
      mo_Log.Lines.Add('Media status : Detected media is full and can''t be erased');
    HWE_NO_SUITABLE_MEDIA:       // Detected media is NOT suitable for the writing
      mo_Log.Lines.Add('Media status : Detected media is NOT suitable for the writing mode required');
  end;

  mo_Log.Lines.Add('');
  mo_Log.Lines.Add(Format('Number of Writers: %d',[NumberOfWriters]));
  mo_Log.Lines.Add(Format('Number of Readers: %d',[NumberOfReaders]));
  mo_Log.Lines.Add('');
  mo_Log.Lines.Add('Writers List:');
  mo_Log.Lines.AddStrings(WriterList);
  mo_Log.Lines.Add('');
  mo_Log.Lines.Add('Device List:');
  mo_Log.Lines.AddStrings(DeviceList);
  mo_Log.Lines.Add('');
  mo_Log.Lines.Add('Write Speed:');
  mo_Log.Lines.AddStrings(WriteSpeedList);
  mo_Log.Lines.Add('');
  mo_Log.Lines.Add('Media Write Speed:');
  mo_Log.Lines.AddStrings(MediaWriteSpeedList);


  // Destroy Instance
  WriterList.Free;
  DeviceList.Free;
  WriteSpeedList.Free;
  MediaWriteSpeedList.Free;
end;

procedure Tfrm_main.mi_WriteDataClick(Sender: TObject);
var
  Path: widestring;
begin
  if not DirectoryExists(BFF.Directory) then
    BFF.Directory := GetCurrentDir;
  BFF.Directory := frm_storage.ReadString('data_folder',BFF.Directory);
  if BFF.Execute then
  begin
    frm_storage.WriteString('data_folder',Bff.Directory);
    // Compute Path
    Path := BFF.Directory;

    // Burn Engine Calls
    UpdateLog;                    // Update log
    if cbb_writeaffinity.ItemIndex >= 0 then
      Be.DVDRWritingModeAffinity := cbb_writeaffinity.ItemIndex;
    PrepareRealtimeSpeed;
    Be.AsyncDataStart(Path,EdLabel.Text);  // Launch asynchronous burn

    // Update User Interface
    cbb_Writers.Enabled := False;
    cbb_speed.Enabled := False;
    cbb_writeaffinity.Enabled := False;
    SetTab(1);
    SetBurning(true);
//    InitGraph;
  end;
end;

procedure Tfrm_main.LogDiscNotifClick(Sender: TObject);
begin
  Be.LogDiscDetectionStatus :=  LogDiscNotif.Checked;
end;

procedure Tfrm_main.AskEmptymediasize1Click(Sender: TObject);
var
  LogLine: String;
  EMS: LongWord;

begin
  Ems := Be.EmptyMediaSize;
  LogLine := format ('Empty media size : %d - %d MB',[Ems, Ems div 512]);
  mo_log.Lines.Add(LogLine);
end;

procedure Tfrm_main.BeLog(Sender: TObject;
                    Kind: TLog_Kind;
                    Const Text: string;
                    Const Params: Array of const);
Var
  LogLine: String;
begin
  LogLine := Format(Text,Params);
  case Kind of
    LK_DECORATION: LogLine := Format('----: %s',[LogLine]);
    LK_VERBOSE:    LogLine := Format('VERBOSE: %s',[LogLine]);
    LK_INFO:       LogLine := Format('INFO: %s',[LogLine]);
    LK_IMPORTANT:  LogLine := Format('IMPORTANT: %s',[LogLine]);
    LK_HINT:       LogLine := Format('HINT: %s',[LogLine]);
    LK_WARNING:    LogLine := Format('WARNING: %s',[LogLine]);
    LK_ERROR:      LogLine := Format('ERROR: %s',[LogLine]);
    LK_CRITICAL:   LogLine := Format('CRITICAL ERROR: %s',[LogLine]);
    LK_DEBUG:      LogLine := Format('DEBUG1: %s',[LogLine]);
    else LogLine := Format('UNKNOWN: %s',[LogLine]);
  end;
  mo_log.Lines.Add(LogLine);
end;

end.
