unit burn.command_interface;

interface

type
  // SCSI Command interface, provided by API in the writing thread to pass some special SCSI command.
  // This can be used to provide
  IVSOBurningAPISCSICommand = interface
    ['{69CE7C00-CC89-4ED5-868B-86E220469A9D}']
    // Execute a SCSI command
    function SendSCSICommand(
      var CDB;                          // Command Descriptor Block (6-16 bytes)
      Send_Data: LongBool = False;      // TRUE if we send data to the device, FALSE to receive data from the device
      Data: Pointer = Nil;              // Pointer to data to send/receive. Can be null for command without data
      DataLen: Integer = 0;             // Length of data to send/receive. 0 if no data is to be sent/received
      SenseBuf: PByte = Nil             // Sense buffer (18 bytes). In case of error if sense buffer is provided the sense data returned by the drive is provided here
    ): LongBool; StdCall;               // Return TRUE in case of success, otherwise FALSE
  end;


  IVSOBurningAPIDataReader = interface
    ['{C01F62C0-A7CE-4851-A080-006A8B1A4629}']
    function GetTotalSectorCount: Cardinal; stdcall;
    procedure readData(
      pData: pointer;                     // pointer to the data buffer
      sector_count: integer;              // number of sectors (2048 bytes) to read
      sector_index: Int64;                // Index of the current sector
      var last: LongBool); stdcall;       // True if there is no more data to read
  end;

  EBurnOp = (burn_idle,                   // burn not started yet
             burn_prepare,                // preparing data structures
             burn_init_device,            // initializing device for writing
             burn_write,                  // writing data
             burn_finishing,              // terminating write
             burn_success,                // burn completed successfully
             burn_error                   // burn completed with error
            );

  IVSOBurningSDKProgress = interface
    ['{BC9BF00C-9F15-43F6-97B0-8962A47032F2}']
    function GetCurrentOperation: EBurnOp;       stdcall; // current burning operation
    function GetTotalDataToWrite: Cardinal;      stdcall; // number of sectors to write
    function GetDataWritten: Integer;            stdcall; // how many sector written so far
    function GetDeviceCacheOccupation(var CacheSize, Occupation: Cardinal): longbool; stdcall; // Actual number of sectors in device cache
    function GetWriteTimings(                    // Return the write timings.
      blockSize: Cardinal = 32;                  // Size of 1 timing entry, in number of sectors. Must be an integer multiple of 32
      blockOffset: Cardinal = 0;                 // First block
      nbBlocks: Cardinal = 0;                    // Number of allocated entries in TimeTable
      TimeTable: PInt64 = nil                    // Pointer to time table. If allocated must contain room for at least NbBlocks entries
                                                 // Each entry is a time in Microsecond since the first sector written
    ): Integer; stdcall;                         // >= 0 : the actual total number of timing blocks available. < 0 : error code

    function GetCurrentDataSpeedKBPS(out CurrentSector: integer; Avg_sec: single = 8): single; stdcall;
  end;

implementation

end.
