﻿unit burn.legacy_enginebase;
// Enum types uses 4 bytes

{$Z4}


interface
uses
  Classes,
  log.interfaces,
  burn.command_interface;

//Data burn parameters
const
  HWE_PARAM_FILELIST = '#HWEFILELIST';
  HWE_PARAM_PRESERVESOURCEPATH = '#HWEPRESERVESOURCEPATH';
  HWE_PARAM_ALWAYSUSEPATHNAME  = '#HWEALWAYSUSEPATHNAME';

// Layer status
const
   HWE_OK = 0;
   HWE_ACCESS_LAYER_CANT_CONNECT = 1;
   HWE_NO_DVD_WRITERS = 2;

// Media status
const
   HWE_NO_SELECTED_DRIVE = -1;  // No selected writers available
   HWE_READY_TO_WRITE = 0;      // Empty DVD (re)writable Media is inserted - Write can start
   HWE_FULL_RW = 1;             // Full rewritable DVD - write can start, but existing data on DVD will be lost
   HWE_NOT_READY = 2;           // Drive is not ready (no media detected)
   HWE_MEDIA_NOT_WRITABLE = 3;  // Detected media is full and can''t be erased
   HWE_NO_SUITABLE_MEDIA = 4;   // Detected media is NOT suitable for the given operation
   HWE_PARSING_DISC = 5;        // A media has been inserted, and is being parsed
   HWE_UNSUPPORTED_MEDIA = 6;   // Media is not supported (the burner can't burn that disc)

// Write status
const
   HWS_INITOK = 0;              // Write init ok - write process has started
   HWS_NOFILES = 1;             // Passed folder contained no files
   HWS_MISSINGIFOORBUP = 2;     // IFO / BUP are missing
   HWS_BADIFO = 3;              // IFO contains invalid data. Will not work on some dvd players
   HWS_CANTCHECKIFO = 4;        // Access to files can't be gained
   HWS_TOOBIG = 5;              // Files are too big to fit on a DVD+/-R(W)
   HWS_BADMEDIASTATUS = 6;      // Media is not ready to be written (see above)
   HWS_BLANKMEDIAFAILED = 7;    // Erasing a RW media has failed
   HWS_NOMORESPACEFORTEMPFILE = 8; // There is not enough space in temp folder to setup the temp file
   HWS_CANTBUILDIMAGE = 9;      // Image can''t be built - lot of cause, including impossibility to create a temp file. in temp folder
   HWS_FORMATFAILED = 10;       // Formatting media has failed
   HWS_DEVICEBUSY = 11;         // Device is busy
   HWS_DEVICEINITFAILED = 12;   // initialization of the writing device has failed
   HWS_STARTBUFFERFAILED = 13;  // Failed to start the data buffering. Probably a windows out of memory
   HWS_BADMEDIA = 14;           // Media is bad
   HWS_FINALIZATIONFAILED = 15; // Finalization process has failed
   HWS_FAILED_WRITING = 16;      // Writing failed (write error in the middle of the write sequence)
   HWS_WRITINGCOMPLETE = 17;    // Write is complete
   HWS_KEYCHECK = 18;           // Unlock key check has failed
   HWS_FORMATCOMPLETE = 19;     // Format media is complete
   HWS_UNSUPPORTED = 20;        // Unsupported write method
   HWS_INVALIDISOPATH = 21;     // Invalid path for ISO creation

   // Extended write failure
   HWS_USER_ABORT = 21;         // User has requested to stop the write sequence
   HWS_FAILED_CALIBRATING = 22; // Write failed at the beginning of the media
   HWS_FAILED_FINISHING = 23;   // Write failed at the end of the media
   HWS_WRITING = 24;            // Writing now

// Write operations
type

  TWriteOperation=(
  WRO_BROWSE_FILES = 0,                 // Files are being browsed
  WRO_COPY_DATA_TO_CACHE = 1,           // Files are being copied to cache
  WRO_DISC_ACCEPTED = 2,                // A new disc has been inserted
  WRO_DISC_REJECTED = 3,                // The inserted disc cannot be used
  WRO_ERASING = 4,                      // Disc is being erased
  WRO_FORMATTING = 5,                   // Disc is being formatted
  WRO_WRITING = 7,                      // Disc is being written
  WRO_FINISIHING = 8,                   // Disc is being finished
  WRO_FINISHED = 9,                     // Disc is finished
  WRO_LOG = 10,                         // A log entry is being sent
  WRO_DEVICE_COUNT_CHANGED = 11,        // Number of drives has changed (hotplugable USB or 1394 drive added/removed)
  WRO_PARSING_DISC = 12                 // Disc is being parsed
  );

type
  HweDriveStatus=(
    DRIVESTATUS_READY        = 0,     // Disc is loaded in the drive and drive is ready
    DRIVESTATUS_NODISC       = 1,     // No disc is loaded in the drive and tray is closed.
    DRIVESTATUS_NOTREADY     = 2,     // Drive is not ready. Disc loading may be in progress.
    DRIVESTATUS_TRAYOPEN     = 3,     // Tray is opened
    DRIVESTATUS_INUSE        = 4,     // Drive is currently in use by another Patin-couffin App.
    DRIVESTATUS_NOTFOUND     = 5,     // There is no drive found matching the DOS letter passed
    DRIVESTATUS_UNKNOWN      = 10);   // Disc status is unknown.

type
  THweBurnMode = (BM_ERASE,         // Erase a media
                  BM_DVDVIDEO,      // Produce a Video DVD
                  BM_DATA,          // Produce a backup of data to archive
                  BM_AUDIO,         // Produce a CD Audio
                  BM_ISOIMAGE,      // Produce a Data media
                  BM_STREAM,        // Produce a Data media with a data callback
                  BM_BLURAYVIDEO);  // Produce a Blu-Ray video

  TDiscRejectionReason = (
    DRR_UNKNOWN,                                  // Disc rejected for unknown reason
    DRR_DISC_NOT_EMPTY,                           // Inserted disc is not empty and not (re)writable
    DRR_DISC_NOT_READABLE,                        // Inserted disc cannot be read (unknown cause)
    DRR_PARSING_INTERRUPTED_NO_DISC,              // No disc detected in drive, and parsing operation was interrupted by user.
    DRR_PARSING_INTERRUPTED_TRAY_OPENED,          // Device's Tray was opened interrupting the parsing operation
    DRR_NO_DISC,                                  // No disc detected in drive (there's nothing loaded in the drive)
    DRR_NO_DISC_TRAY_OPENED,                      // No disc detected in drive and Tray is opening/opened
    DRR_INCOMPATIBLE_DISC,                        // Disc is not compatible with the drive (ex. a DVD inserted in a CD-Rom only reader)
    DRR_INCOMPATIBLE_WRITE,                       // Disc is compatible with the drive but cannot be written (ex. a DVD+RW with a writing speed that doesn't support the drive)
    DRR_HARDWARE_FAILURE,                         // Drive has reported an hardware failure while parsing disc. This denotes a poor disc or a drive issue
    DRR_WRONG_DISC_TYPE                           // Disc type isn't correct (ex: inserting a CD-R when you need to write a DVD-VIDEO)
  );
{
  THweBurnReadyStatus=(
    BURN_NO_SELECTED_DRIVE,         // No drive selected
    BURN_PARSING_DISC,              // Disc being parsed
    BURN_DISC_NOT_WRITABLE,         // Disc is not writable (It's a ROM or no longer writable disc)
    BURN_DISC_UNSUPPORTED,          // Unsupported disc (The burner doesn't know how to burn that disc)
    BURN_DISC_NOT_SUITABLE,         // Disc is not correct for the burning target
    BURN_CAN_START,                 // Burn can start
    BURN_NEED_ERASE);               // Burn can start and need erasure
}
const
    HWE_WRITE_AFFINITY_AUTOMATIC = 0;        // Let the engine detect automatically which writing mode is more suitable
                                             // The decision is done according to:
                                             // * If the source is mono-session, or if trying to burn a ISO image, DAO is tried first.
                                             // * If the source is multi-session or there is existing data on DVD, TAO is tried first.
                                             // * for each burning mode, there is a BLACKLIST of drives that doesn't support one or the other mode
                                             //   if such drive is encountered, the supported mode is forced
                                             // * if DAO has to be used with a multisession source, it'll turn multisession off while issuing a warning about it.
                                             //   if data already exist on a DVD-R in multisession, TAO mode is always forced, and DAO will not even be tried.

    // For all the following modes, If DAO is attempted and multimedia mode is on, it is turned off and a warning is issued
    HWE_WRITE_AFFINITY_FORCE_DAO = 1;        // Override all aforementionned logic to force DAO writing mode, if DAO fail to initialize, burning will fail
    HWE_WRITE_AFFINITY_FORCE_TAO = 2;        // Override all aforementionned logic to force TAO writing mode, if TAO fail to initialize, burning will fail
    HWE_WRITE_AFFINITY_PREFER_DAO = 3;       // prefer DAO writing mode, if DAO fail to initialize, log failure and try TAO
    HWE_WRITE_AFFINITY_PREFER_TAO = 4;       // prefer TAO writing mode, if TAO fail to initialize, log failure and try DAO






const
   burn_engine_log_group = 'Burn engine';
   MEDIATYPE_UNKNOWN    = $0000;   // No profile
   MEDIATYPE_NRDISC     = $0001;   // Non removable disc
   MEDIATYPE_RMDISC     = $0002;   // Removable Disc
   MEDIATYPE_MOE        = $0003;   // Magneto Optical Erasable
   MEDIATYPE_OPTICWO    = $0004;   // Optical Write Once
   MEDIATYPE_ASMO       = $0005;   // Advanced Storage - Magneto Optical
   MEDIATYPE_CDROM      = $0008;   // CD-ROM (not writable)
   MEDIATYPE_CDR        = $0009;   // CD-R (Write once)
   MEDIATYPE_CDRW       = $000A;   // CD-RW (R/W)
   MEDIATYPE_DVDROM     = $0010;   // DVD-ROM (not writable)
   MEDIATYPE_DVDR       = $0011;   // DVD-R
   MEDIATYPE_DVDRAM     = $0012;   // DVD-RAM
   MEDIATYPE_DVDRW      = $0013;   // DVD-RW Restricted overwrite
   MEDIATYPE_DVDRWSEQ   = $0014;   // DVD-RW Sequential Recording
   MEDIATYPE_DLDVDR     = $0015;   // dual Layer DVD-R Sequential Recording
   MEDIATYPE_DLDVDRJMP  = $0016;   // dual Layer DVD-R Jump Recording
   MEDIATYPE_DVDPLUSRW  = $001A;   // DVD+RW
   MEDIATYPE_DVDPLUSR   = $001B;   // DVD+R
   MEDIATYPE_DDCDROM    = $0020;   // Double Density CD-ROM
   MEDIATYPE_DDCDR      = $0021;   // Double Density CD-R (Write once)
   MEDIATYPE_DDCDRW     = $0022;   // Double Density CD-RW
   MEDIATYPE_DLDVDPLUSRW= $002A;   // Dual Layer DVD+RW
   MEDIATYPE_DLDVDPLUSR = $002B;   // Dual Layer DVD+R
   MEDIATYPE_HDBURNCDROM= $0080;   // HDBurn CD-R
   MEDIATYPE_HDBURNCDR  = $0081;   // HDBurn CD-R
   MEDIATYPE_HDBURNCDRW = $0082;   // HDBurn CD-RW
   MEDIATYPE_BDROM        = $0040;   // BluRay Disc ROM
   MEDIATYPE_BDRS         = $0041;   // BluRay Disc Sequential Recording
   MEDIATYPE_BDRR         = $0042;   // BluRay Disc Random Recording
   MEDIATYPE_BDRE         = $0043;   // BluRay Disc Random Recording
   MEDIATYPE_HD_DVDROM    = $0050;   // HD DVDROM
   MEDIATYPE_HD_DVDR      = $0051;   // HD DVD-R
   MEDIATYPE_HD_DVDRAM    = $0052;   // HD DVD-RAM




type
  PSingleArray = ^TSingleArray;
  TSingleArray = array [0..0] of Single;

type
  PInt64Array = ^TInt64Array;
  TInt64Array = array[0..maxint div sizeof(int64) - 1] of int64;

type
  THWENotifyEvent = procedure (Sender: TObject; WriteOp: Longword; Progress: Longword) of object;


  TDataCallback   = procedure (user_opaque: pointer; sector_index: integer; data: pbyte; data_bytesize: integer; Device: IVSOBurningAPISCSICommand); StdCall;

Type
  TAbstractHeadlessEngine = class
  protected
    function GetUserLayerBreak: integer; virtual; Abstract;
    procedure SetUserLayerBreak(const Value: integer); virtual; Abstract;
    function GetMediaStatus: Integer; virtual; Abstract;
    function GetWriterCount: integer; virtual; Abstract;
    function GetReaderCount: integer; virtual; Abstract;
    function GetWriteStatus: Integer; virtual; Abstract;
    function GetLayerStatus: Integer; virtual; Abstract;
    function GetMediaType: Integer; virtual; Abstract;
    function GetWriterName(Index: Integer): String; virtual; Abstract;
    function GetOnNotify: THWENotifyEvent; virtual; Abstract;
    procedure SetOnNotify(const Value: THWENotifyEvent); virtual; Abstract;
    function GetEnablePolling: Boolean; virtual; Abstract;
    procedure SetEnablePolling(const Value: Boolean); virtual; Abstract;
    function GetSendWebStats: boolean; virtual; abstract;
    procedure SetSendWebStats(const Value: boolean); virtual; abstract;
    function GetDontAutoEject: boolean; virtual; abstract;
    procedure SetDontAutoEject(const Value: boolean); virtual; abstract;
    function GetPreferDVDRSAO: boolean; virtual; abstract;
    procedure SetPreferDVDRSAO(const Value: boolean); virtual; abstract;
    function GetCheckAndFixIfo: boolean; virtual; abstract;
    procedure SetCheckAndFixIfo(const Value: boolean); virtual; abstract;
    function GetDVDRWritingModeAffinity: Longword; virtual; abstract;
    procedure SetDVDRWritingModeAffinity(const Value: Longword); virtual; abstract;
    function GetBurnMode: THweBurnMode; virtual; abstract;
    function getIsoPath: String; virtual; Abstract;
    procedure SetIsoPath(const Value: String); virtual; Abstract;
    function GetStrictMediaEnforcment: boolean; virtual; abstract;
    procedure SetStrictMediaEnforcment(const Value: boolean); virtual; abstract;
    function GetHookProgramActive: boolean; virtual; abstract;
    function GetLogDiscDetectionStatus: boolean; virtual; abstract;
    procedure SetLogDiscDetectionStatus(const Value: boolean); virtual; abstract;
  public

     procedure GetWriterList(S:TStrings); virtual; Abstract;
     procedure GetDeviceList(S:TStrings); virtual; Abstract;

     procedure AsyncStart(Folder, VolumeName: String); virtual; Abstract;
     procedure AsyncBdvStart(Folder, VolumeName: String); virtual; Abstract;
     procedure AsyncDataStart(Folder, VolumeName: String); virtual; Abstract;
     procedure AsyncISOStart(IsoPath: String); virtual; Abstract;
     procedure AsyncBlank(EjectOnSuccess: boolean; EjectOnFail: Boolean); virtual; Abstract;
     procedure AsyncBinaryStart(aDataStream: IVSOBurningAPIDataReader); virtual; Abstract;

     procedure SetWriter(Index: Integer); virtual; Abstract;
     procedure AbortQuery; virtual; Abstract;

     function SendKey(aKey: Pointer; Len: Integer): boolean; virtual; Abstract;
     function OpenTray: Boolean; virtual; Abstract;
     function CloseTray: Boolean; virtual; Abstract;
     function LockTray: Boolean; virtual; Abstract;
     function UnlockTray: Boolean; virtual; Abstract;

     function MediaVolumeName: string; virtual; Abstract;
     function EmptyMediaSize: longword; virtual; Abstract;
     function SuitableMediaList(SectorsToWrite: Longword): String; virtual; Abstract;

     function GetWriterMode(PlusRw: Boolean): Byte; virtual; Abstract;
     function SetWriterMode(PlusRw, ToRom: Boolean):Boolean; virtual; Abstract;
     function SetMediaToRom(ToRom: Boolean):Boolean; virtual; Abstract;

     function GetDriveStatus(DriveLetter: AnsiChar): HweDriveStatus; virtual; Abstract;

     function DeviceName(C: AnsiChar): string; virtual; Abstract;
     function SetWriterByDosDevice(C: AnsiChar): boolean; virtual; Abstract;
     function GetWriterDosDevice(dwIndex: Integer): AnsiChar; virtual; Abstract;
     function DmaEnabled(C:AnsiChar): integer; virtual; abstract;
     function DmaName(C:AnsiChar): string; virtual; abstract;

     function GetSupportedMediaList: string; virtual; Abstract;
     function IsDvdWriter: Boolean; virtual; abstract;
     function SetAnyDVDActive(Active : boolean): Boolean; virtual; abstract;
     function SetDataCallback(Rcb: TDataCallback; Rcbo: Pointer): boolean; virtual; abstract;



     property WriterCount: integer read GetWriterCount;
     property ReaderCount: integer read GetReaderCount;
     property LayerStatus: Integer read GetLayerStatus;
     property MediaStatus: Integer read GetMediaStatus;
     property WriteStatus: Integer read GetWriteStatus;
     property BurnMode: THweBurnMode read GetBurnMode;
     property EnablePolling: Boolean read GetEnablePolling write SetEnablePolling;
     property SendWebStats: boolean read GetSendWebStats write SetSendWebStats;
     property DontAutoEject: boolean read GetDontAutoEject write SetDontAutoEject;
     property PreferDVDRSAO: boolean read GetPreferDVDRSAO write SetPreferDVDRSAO;
     property DVDRWritingModeAffinity: Longword read GetDVDRWritingModeAffinity write SetDVDRWritingModeAffinity;
     property CheckAndFixIfo: boolean read GetCheckAndFixIfo write SetCheckAndFixIfo;
     property UserLayerBreak: integer read GetUserLayerBreak write SetUserLayerBreak;
     property StrictMediaEnforcment: boolean read GetStrictMediaEnforcment write SetStrictMediaEnforcment;
     property IsoPath: String read getIsoPath write SetIsoPath;
     property HookProgramActive: boolean read GetHookProgramActive;

     property SupportedMediaList: String read GetSupportedMediaList;
     property LogDiscDetectionStatus: boolean read GetLogDiscDetectionStatus write SetLogDiscDetectionStatus;

     property MediaType: Integer read GetMediatype;
     property WriterName[Index: Integer]: string read GetWriterName;
     property OnNotify: THWENotifyEvent read GetOnNotify write SetOnNotify;
  End;

Type
  TBurningEngine = class(TAbstractHeadlessEngine)
  private
    Hwe: Longword;
    FOnNotify: THWENotifyEvent;
    FOnLog: TLogAddFormatedEvent;
    FHWe_Available: boolean;
    FWriteAllowed: Boolean;
    FCancelAllowed: Boolean;
    FBurnMode: THweBurnMode;
    procedure DoNotify(WriteOp: Longword; Progress: NativeUint);
    procedure SetOnLog(const Value: TLogAddFormatedEvent);
    function GetBlankAllowed: Boolean;
    function GetDll_Details: String;
  protected
    function GetBurnMode: THweBurnMode; override;
    function GetUserLayerBreak: integer; override;
    procedure SetUserLayerBreak(const Value: integer); override;
    function GetEnablePolling: Boolean; override;
    procedure SetEnablePolling(const Value: Boolean); override;
    function GetSendWebStats: boolean; override;
    procedure SetSendWebStats(const Value: boolean); override;
    function GetLayerStatus: Integer; override;
    function GetMediaStatus: Integer; override;
    function GetMediaType: Integer; override;
    function GetOnNotify: THWENotifyEvent; override;
    procedure SetOnNotify(const Value: THWENotifyEvent); override;
    function GetReaderCount: Integer; override;
    function GetWriterCount: Integer; override;
    function GetWriteStatus: Integer; override;
    function GetWriterName(Index: Integer): String; override;
    function GetDontAutoEject: boolean; override;
    procedure SetDontAutoEject(const Value: boolean); override;
    function GetPreferDVDRSAO: boolean; override;
    procedure SetPreferDVDRSAO(const Value: boolean); override;
    function GetCheckAndFixIfo: boolean; override;
    procedure SetCheckAndFixIfo(const Value: boolean); override;
    function GetDVDRWritingModeAffinity: Longword; override;
    procedure SetDVDRWritingModeAffinity(const Value: Longword); override;
    function getIsoPath: String; override;
    procedure SetIsoPath(const Value: String); override;
    function GetStrictMediaEnforcment: boolean; override;
    procedure SetStrictMediaEnforcment(const Value: boolean); override;
    function GetHookProgramActive: boolean; override;
    function GetLogDiscDetectionStatus: boolean; override;
    procedure SetLogDiscDetectionStatus(const Value: boolean); override;


  public
//     FIsc: PHweInputStreamContext;

     constructor Create(Const Path: String; const WndH: THandle=0);
     Destructor Destroy; Override;

     procedure GetWriterList(S:TStrings); override;
     procedure GetDeviceList(S:TStrings); override;

     procedure SetLogfile(LFN: String);

     procedure AsyncStart(Folder, VolumeName: String); override;
     procedure AsyncDataStart(Folder, VolumeName: String); override;
     procedure AsyncISOStart(IsoPath: String); override;
     procedure AsyncBlank(EjectOnSuccess: boolean=false; EjectOnFail: Boolean=true); override;
//     function AsyncStreamStart(AReadFunc: Pointer; Size: Integer; Stats: Boolean): PHweInputStreamContext; // Deprecated and removed. Replaced by Async
     procedure AsyncBdvStart(Folder, VolumeName: String); override;
     procedure AsyncBinaryStart(aDataStream: IVSOBurningAPIDataReader); override;

     procedure SetWriter(Index: Integer); override;
     procedure AbortQuery; override;

     function SendKey(aKey: Pointer; Len: Integer): boolean; override;

     function OpenTray: Boolean; override;
     function CloseTray: Boolean; override;
     function LockTray: Boolean; override;
     function UnlockTray: Boolean; override;

     function MediaVolumeName: string; override;
     function EmptyMediaSize: Longword; override;
     function SuitableMediaList(SectorsToWrite: Longword): String; override;


     function GetWriterMode(PlusRw: Boolean): Byte; override;
     function SetWriterMode(PlusRw, ToRom: Boolean):Boolean; override;
     function SetMediaToRom(ToRom: Boolean):Boolean; override;

     function GetDriveStatus(DriveLetter: AnsiChar): HweDriveStatus; override;

     function DeviceName(C: AnsiChar): string; override;
     function SetWriterByDosDevice(C: AnsiChar): boolean; override;
     function GetWriterDosDevice(dwIndex: Integer): AnsiChar; override;
     function DmaEnabled(C:AnsiChar): integer; override;
     function DmaName(C:AnsiChar): string; override;


     function IsDvdWriter: Boolean; override;
     function SetAnyDVDActive(Value : Boolean):boolean;override;
     function GetSupportedMediaList: string; override;
     function SetDataCallback(Rcb: TDataCallback; Rcbo: Pointer): boolean; override;
     function GetSDKVersionInformation(out infos: string): boolean;
     function GetProgressInterface(out intf: IVSOBurningSDKProgress): boolean;


     procedure GetWriteSpeeds(S: TStrings);

     procedure SetWriteSpeed(S: TStrings;Index: Integer);

     procedure GetMediaWriteSpeeds(S: TStrings);

     property OnLog: TLogAddFormatedEvent read FOnLog write SetOnLog;
     property Hwe_Available: boolean read FHWe_Available;
     property WriteAllowed: Boolean read FWriteAllowed;
     property BlankAllowed: Boolean read GetBlankAllowed;
     property CancelAllowed: Boolean read FCancelAllowed;
     property Dll_Details: String read GetDll_Details;
  End;

  function GetMediaName(aMediaType: Word): String;
  function GetNumberOfSectorsOfEmptyMedia(aProfile: Word): Int64;
  function IntToMo(Value : Int64) : string;

  function LoadHwe: boolean;
  function HweLoaded: Boolean;
  procedure FreeHwe;


implementation

Uses
  SysUtils,
  Windows;

const
    STR_bytes       : String =  'b.';
    STR_KBytes      : String = 'Kb.';
    STR_MBytes      : String = 'Mb.';
    STR_GBytes      : String = 'GB.';
    STR_TBytes      : String = 'Tb.';

    HWE_DLL_NAME  = 'vso_hwe.dll';

var
  Hwe_Handle: THandle = 0;
  HweLoadCount: Integer = 0;
  Hwe_Init: function (Const Path: PAnsiChar; const WndH: THandle; Var H: Longword): Integer; StdCall;
  Hwe_Free: procedure (const H: Longword); StdCall;
  Hwe_GetDvdWritersCount: function (const H: Longword): Longword; StdCall;
  Hwe_GetDvdReadersCount: function (const H: Longword): Longword; StdCall;
  Hwe_GetDvdWriterName: function (const Hwe: Longword; dwIndex: integer; szName: PAnsiChar; dwSize: Longword): LongBool; StdCall;
  Hwe_SetWriter: function (const Hwe: Longword; dwIndex: integer): LongBool; StdCall;
  Hwe_GetLayerStatus: function (const Hwe: Longword): Integer; StdCall;
  Hwe_GetMediaStatus: function (const Hwe: Longword): Integer; StdCall;
  Hwe_GetWriteStatus: function (const Hwe: Longword): Integer; StdCall;
  Hwe_GetMediaType: function (const Hwe: Longword): Integer; StdCall;
  Hwe_StartWriting: procedure (const Hwe: Longword; szDvdPath, szVolumeName: PAnsiChar); StdCall;
  Hwe_StartDataWriting: procedure (const Hwe: Longword; szDvdPath, szVolumeName: PAnsiChar); StdCall;
  Hwe_StartIsoWriting: procedure (const Hwe: Longword; szIsoPath: PAnsiChar); StdCall;
  Hwe_Blank: procedure (const Hwe: Longword; EjectOnSuccess: longbool; EjectOnFail: longbool); StdCall;
  Hwe_RegisterCallback: procedure (const Hwe: Longword; CallbackProc: Pointer; UserParam: NativeUint); StdCall;
  // CallbackProc  = procedure (Hwe: NativeUint; WriteOp: Longword; Progress: NativeUint; UserParam: NativeUint); StdCall;
  Hwe_AbortWrite: procedure (const Hwe: Longword); StdCall;
  Hwe_CloseTray: function (const Hwe: Longword): longbool; stdcall;
  Hwe_OpenTray: function (const Hwe: Longword): longbool; stdcall;
  Hwe_EmptyMediaSize: function (const Hwe: Longword): longword; stdcall;
  Hwe_SendKey: function (const Hwe: Longword; const Key: pointer; len: integer): longbool; stdcall;
  Hwe_GetDriveStatus: function (const Hwe: Longword; DriveLetter: AnsiChar): integer; stdcall;
  Hwe_GetDeviceName: function (const Hwe: Longword; DosLetter: AnsiChar; szName: PAnsiChar; dwSize: Longword): LongBool; StdCall;
  Hwe_SetWriterBydosDevice: function (const Hwe: Longword; DosLetter: AnsiChar): LongBool; StdCall;
  Hwe_GetWriterDosDevice: function (const Hwe: Longword; dwIndex: integer): AnsiChar; StdCall;
  Hwe_GetMediaVolumeName: function (const Hwe: Longword; szName: PAnsiChar; dwSize: integer): integer; stdcall;
  Hwe_LockTray: function (const Hwe: Longword): longbool; stdcall;
  Hwe_UnlockTray: function (const Hwe: Longword): longbool; stdcall;


  Hwe_SetMediaToRom: function (const Hwe: Longword; ToRom: LongBool): Longbool; stdcall;
  Hwe_GetWriterMode: function (const Hwe: Longword; PlusRW: LongBool): Longword; stdcall;
  Hwe_SetWriterMode: function (const Hwe: Longword; PlusRW, ToRom: LongBool): Longbool; stdcall;

  Hwe_SetLogFile: procedure(const H: Longword; const LogfilePath: pAnsiChar); stdcall;

  Hwe_GetWriteSpeeds: function (const Hwe: Longword; SpeedList: PIntegerArray; EntryCount: Integer): longbool; stdcall;
  Hwe_SetWriteSpeed: function (const Hwe: Longword; Speed: Integer): longbool; stdcall;
  Hwe_GetMediaWriteSpeeds: function (const Hwe: Longword; SpeedList: PSingleArray; EntryCount: Integer): longbool; stdcall;

  Hwe_GetEnablePolling: function (const Hwe: Longword): longbool; stdcall;
  Hwe_SetEnablePolling: procedure (const Hwe: Longword; const Poll: Longbool); stdcall;

  Hwe_GetSendWebStats: function (const Hwe: Longword): longbool; stdcall;
  Hwe_SetSendWebStats: procedure (const Hwe: Longword; const SendStats: Longbool); stdcall;

  Hwe_GetDontAutoEject: function (const Hwe: Longword): longbool; stdcall;
  Hwe_SetDontAutoEject: procedure (const Hwe: Longword; const Value: Longbool); stdcall;


//  Hwe_StartStreamWriting: procedure (const Hwe: Longword; WriteStream: PHweInputStreamContext); stdcall;
  Hwe_GetWriterSupportedMediaList: function (const Hwe: Longword; szName: PAnsiChar; dwSize: integer): integer; stdcall;

  Hwe_IsDvdWriter: function (const Hwe: Longword): Longbool; stdcall;
  Hwe_SetAnyDVDActive: function (const Hwe: Longword;const Value: Longbool): Longbool; stdcall;

  Hwe_GetPreferDVDRSao: function (const Hwe: Longword): longbool; stdcall;
  Hwe_SetPreferDVDRSao: procedure (const Hwe: Longword; const Value: Longbool); stdcall;

  Hwe_GetCheckAndFixIfo: function(const Hwe: Longword): longbool; stdcall;
  Hwe_SetCheckAndFixIfo: procedure(const Hwe: Longword; const Value: Longbool); stdcall;

  Hwe_GetLogDiscDetectionStatus: function(const Hwe: Longword): longbool; stdcall;
  Hwe_SetLogDiscDetectionStatus: procedure(const Hwe: Longword; const Value: Longbool); stdcall;

  Hwe_GetSuitableMediaList: function (const Hwe: Longword; SizeToWrite: Int64; szList: PAnsiChar; dwSize: integer): integer; stdcall;

  Hwe_DmaEnabled: function (const Hwe: Longword; DriveLetter: AnsiChar): integer; stdcall;
  Hwe_DmaName: function (const Hwe: Longword; DriveLetter: AnsiChar; szName: PAnsiChar; dwSize: integer): integer; stdcall;

  Hwe_SetLayerBreak: procedure (const Hwe: Longword; Value: Integer); stdcall;
  Hwe_GetLayerBreak: function (const Hwe: Longword): integer; stdcall;

  Hwe_GetDVDRWritingModeAffinity: function(const Hwe: Longword): LongWord; stdcall;
  Hwe_SetDVDRWritingModeAffinity: procedure(const Hwe: Longword; const Value: Longword); stdcall;

  Hwe_StartBdvWriting: procedure (const Hwe: Longword;
                                        szDvdPath,
                                        szVolumeName: PAnsiChar); StdCall;

  Hwe_SetIsoPath: procedure (const Hwe: Longword; szPath: PAnsiChar); StdCall;
  Hwe_GetIsoPath: function (const Hwe: Longword; szPath: PAnsiChar; dwSize: Longword): Longword; StdCall;

  Hwe_GetStrictMediaEnforcment:function (const Hwe: Longword): longbool; stdcall;
  Hwe_SetStrictMediaEnforcment: procedure (const Hwe: Longword; Value: Longbool); stdcall;

  Hwe_IsHookProgramActive: function(const Hwe: Longword): Longbool; stdcall;
//  Hwe_SetDataCallback: function(const Hwe: Longword; data_cb: TDataCallback; user_opaque: Pointer): longbool; stdcall;

  Hwe_GetSDKVersionInformation: function(const Hwe: NativeUint; szInfo: MarshaledAString; pDwInfoLength: PCardinal): longbool; stdcall;

  Hwe_GetBlankAllowed: function(const Hwe: NativeUint): longbool; stdcall;

  Hwe_StartBinaryWriting: function (const Hwe: NativeUInt; ReadStream: IVSOBurningAPIDataReader): longbool; stdcall;

  Hwe_GetProgressInterface: function (const Hwe: NativeUInt; out intf: IVSOBurningSDKProgress): longbool; stdcall;



function LoadHwe: boolean;
begin
  Result := HweLoaded;
  if result then
  begin
    inc(HweLoadCount);
    Exit;
  end;
  try
//    Hwe_Handle := LoadLibrary(HWE_DLL_NAME);
    Hwe_Handle := SafeLoadLibrary(HWE_DLL_NAME,SEM_NOOPENFILEERRORBOX);
    HweLoadCount := 1;
    Result := Hwe_Handle > 32;
    if not result then
    begin
      Hwe_Handle := 0;
      exit;
    End;
    try
      @Hwe_Init := GetProcAddress(Hwe_Handle,'Hwe_Init'); Assert (@Hwe_Init <> Nil);
      @Hwe_Free := GetProcAddress(Hwe_Handle,'Hwe_Free'); Assert (@Hwe_Free <> Nil);
      @Hwe_GetDvdWritersCount := GetProcAddress(Hwe_Handle,'Hwe_GetDvdWritersCount'); Assert (@Hwe_GetDvdWritersCount <> Nil);
      @Hwe_GetDvdReadersCount := GetProcAddress(Hwe_Handle,'Hwe_GetDvdReadersCount'); Assert (@Hwe_GetDvdReadersCount <> Nil);
      @Hwe_GetDvdWriterName := GetProcAddress(Hwe_Handle,'Hwe_GetDvdWriterName'); Assert (@Hwe_GetDvdWriterName <> Nil);
      @Hwe_SetWriter := GetProcAddress(Hwe_Handle,'Hwe_SetWriter'); Assert (@Hwe_SetWriter <> Nil);
      @Hwe_GetLayerStatus := GetProcAddress(Hwe_Handle,'Hwe_GetLayerStatus'); Assert (@Hwe_GetLayerStatus <> Nil);
      @Hwe_GetMediaStatus := GetProcAddress(Hwe_Handle,'Hwe_GetMediaStatus'); Assert (@Hwe_GetMediaStatus <> Nil);
      @Hwe_GetWriteStatus := GetProcAddress(Hwe_Handle,'Hwe_GetWriteStatus'); Assert (@Hwe_GetWriteStatus <> Nil);
      @Hwe_GetMediaType := GetProcAddress(Hwe_Handle,'Hwe_GetMediaType'); Assert (@Hwe_GetMediaType <> Nil);
      @Hwe_StartWriting := GetProcAddress(Hwe_Handle,'Hwe_StartWriting'); Assert (@Hwe_StartWriting <> Nil);
      @Hwe_StartDataWriting := GetProcAddress(Hwe_Handle,'Hwe_StartDataWriting'); Assert (@Hwe_StartDataWriting <> Nil);
      @Hwe_StartIsoWriting := GetProcAddress(Hwe_Handle,'Hwe_StartIsoWriting'); Assert (@Hwe_StartIsoWriting <> Nil);
      @Hwe_Blank := GetProcAddress(Hwe_Handle,'Hwe_Blank'); Assert (@Hwe_Blank <> Nil);
      @Hwe_RegisterCallback := GetProcAddress(Hwe_Handle,'Hwe_RegisterCallback'); Assert (@Hwe_RegisterCallback <> Nil);
      @Hwe_AbortWrite := GetProcAddress(Hwe_Handle,'Hwe_AbortWrite'); Assert (@Hwe_AbortWrite <> Nil);
      @Hwe_CloseTray := GetProcAddress(Hwe_Handle,'Hwe_CloseTray'); Assert (@Hwe_CloseTray <> Nil);
      @Hwe_OpenTray := GetProcAddress(Hwe_Handle,'Hwe_OpenTray'); Assert (@Hwe_OpenTray <> Nil);
      @Hwe_EmptyMediaSize := GetProcAddress(Hwe_Handle,'Hwe_EmptyMediaSize'); Assert (@Hwe_EmptyMediaSize <> Nil);
      @Hwe_SendKey := GetProcAddress(Hwe_Handle,'Hwe_SendKey'); Assert (@Hwe_SendKey <> Nil);
      @Hwe_GetDriveStatus := GetProcAddress(Hwe_Handle,'Hwe_GetDriveStatus'); Assert (@Hwe_GetDriveStatus <> Nil);
      @Hwe_GetDeviceName := GetProcAddress(Hwe_Handle,'Hwe_GetDeviceName'); Assert (@Hwe_GetDeviceName <> Nil);
      @Hwe_SetWriterBydosDevice := GetProcAddress(Hwe_Handle,'Hwe_SetWriterBydosDevice'); Assert (@Hwe_SetWriterBydosDevice <> Nil);
      @Hwe_GetWriterDosDevice := GetProcAddress(Hwe_Handle,'Hwe_GetWriterDosDevice'); Assert (@Hwe_GetWriterDosDevice <> Nil);
      @Hwe_GetMediaVolumeName := GetProcAddress(Hwe_Handle,'Hwe_GetMediaVolumeName'); Assert (@Hwe_GetMediaVolumeName <> Nil);
      @Hwe_LockTray := GetProcAddress(Hwe_Handle,'Hwe_LockTray'); Assert (@Hwe_LockTray <> Nil);
      @Hwe_UnlockTray := GetProcAddress(Hwe_Handle,'Hwe_UnlockTray'); Assert (@Hwe_UnlockTray <> Nil);
      @Hwe_SetMediaToRom := GetProcAddress(Hwe_Handle,'Hwe_SetMediaToRom'); Assert (@Hwe_SetMediaToRom <> Nil);
      @Hwe_GetWriterMode := GetProcAddress(Hwe_Handle,'Hwe_GetWriterMode'); Assert (@Hwe_GetWriterMode <> Nil);
      @Hwe_SetWriterMode := GetProcAddress(Hwe_Handle,'Hwe_SetWriterMode'); Assert (@Hwe_SetWriterMode <> Nil);
      @Hwe_SetLogFile := GetProcAddress(Hwe_Handle,'Hwe_SetLogFile'); Assert (@Hwe_SetLogFile <> Nil);
      @Hwe_GetWriteSpeeds := GetProcAddress(Hwe_Handle,'Hwe_GetWriteSpeeds'); Assert (@Hwe_GetWriteSpeeds <> Nil);
      @Hwe_SetWriteSpeed := GetProcAddress(Hwe_Handle,'Hwe_SetWriteSpeed'); Assert (@Hwe_SetWriteSpeed <> Nil);
      @Hwe_GetMediaWriteSpeeds := GetProcAddress(Hwe_Handle,'Hwe_GetMediaWriteSpeeds'); Assert (@Hwe_GetMediaWriteSpeeds <> Nil);
      @Hwe_GetEnablePolling := GetProcAddress(Hwe_Handle,'Hwe_GetEnablePolling'); Assert (@Hwe_GetEnablePolling <> Nil);
      @Hwe_SetEnablePolling := GetProcAddress(Hwe_Handle,'Hwe_SetEnablePolling'); Assert (@Hwe_SetEnablePolling <> Nil);
      @Hwe_GetSendWebStats := GetProcAddress(Hwe_Handle,'Hwe_GetSendWebStats'); Assert (@Hwe_GetSendWebStats <> Nil);
      @Hwe_SetSendWebStats := GetProcAddress(Hwe_Handle,'Hwe_SetSendWebStats'); Assert (@Hwe_SetSendWebStats <> Nil);
      @Hwe_GetDontAutoEject := GetProcAddress(Hwe_Handle,'Hwe_GetDontAutoEject'); Assert (@Hwe_GetDontAutoEject <> Nil);
      @Hwe_SetDontAutoEject := GetProcAddress(Hwe_Handle,'Hwe_SetDontAutoEject'); Assert (@Hwe_SetDontAutoEject <> Nil);
//      @Hwe_StartStreamWriting := GetProcAddress(Hwe_Handle,'Hwe_StartStreamWriting'); Assert (@Hwe_StartStreamWriting <> Nil);
      @Hwe_GetWriterSupportedMediaList := GetProcAddress(Hwe_Handle,'Hwe_GetWriterSupportedMediaList'); Assert (@Hwe_GetWriterSupportedMediaList <> Nil);
      @Hwe_IsDvdWriter := GetProcAddress(Hwe_Handle,'Hwe_IsDvdWriter'); Assert (@Hwe_IsDvdWriter <> Nil);
      @Hwe_SetAnyDVDActive  := GetProcAddress(Hwe_Handle,'Hwe_SetAnyDVDActive'); Assert (@Hwe_SetAnyDVDActive <> Nil);
      @Hwe_GetPreferDVDRSao := GetProcAddress(Hwe_Handle,'Hwe_GetPreferDVDRSao'); Assert (@Hwe_GetPreferDVDRSao <> Nil);
      @Hwe_SetPreferDVDRSao := GetProcAddress(Hwe_Handle,'Hwe_SetPreferDVDRSao'); Assert (@Hwe_SetPreferDVDRSao <> Nil);
      @Hwe_GetSuitableMediaList := GetProcAddress(Hwe_Handle,'Hwe_GetSuitableMediaList'); Assert (@Hwe_GetSuitableMediaList <> Nil);
      @Hwe_DmaEnabled := GetProcAddress(Hwe_Handle,'Hwe_DmaEnabled'); Assert (@Hwe_DmaEnabled <> Nil);
      @Hwe_DmaName := GetProcAddress(Hwe_Handle,'Hwe_DmaName'); Assert (@Hwe_DmaName <> Nil);
      @Hwe_SetLayerBreak := GetProcAddress(Hwe_Handle,'Hwe_SetLayerBreak'); Assert (@Hwe_SetLayerBreak <> Nil);
      @Hwe_GetLayerBreak := GetProcAddress(Hwe_Handle,'Hwe_GetLayerBreak'); Assert (@Hwe_GetLayerBreak <> Nil);
      @Hwe_GetCheckAndFixIfo := GetProcAddress(Hwe_Handle,'Hwe_GetCheckAndFixIfo'); Assert (@Hwe_GetCheckAndFixIfo <> Nil);
      @Hwe_SetCheckAndFixIfo := GetProcAddress(Hwe_Handle,'Hwe_SetCheckAndFixIfo'); Assert (@Hwe_SetCheckAndFixIfo <> Nil);
      @Hwe_GetDVDRWritingModeAffinity := GetProcAddress(Hwe_Handle,'Hwe_GetDVDRWritingModeAffinity'); Assert (@Hwe_GetDVDRWritingModeAffinity <> Nil);
      @Hwe_SetDVDRWritingModeAffinity := GetProcAddress(Hwe_Handle,'Hwe_SetDVDRWritingModeAffinity'); Assert (@Hwe_SetDVDRWritingModeAffinity <> Nil);
      @Hwe_StartBdvWriting := GetProcAddress(Hwe_Handle,'Hwe_StartBdvWriting'); Assert (@Hwe_StartBdvWriting <> Nil);
      @Hwe_SetIsoPath := GetProcAddress(Hwe_Handle,'Hwe_SetIsoPath'); Assert (@Hwe_SetIsoPath <> Nil);
      @Hwe_GetIsoPath := GetProcAddress(Hwe_Handle,'Hwe_GetIsoPath'); Assert (@Hwe_GetIsoPath <> Nil);
      @Hwe_GetStrictMediaEnforcment := GetProcAddress(Hwe_Handle,'Hwe_GetStrictMediaEnforcment'); Assert (@Hwe_GetStrictMediaEnforcment <> Nil);
      @Hwe_SetStrictMediaEnforcment := GetProcAddress(Hwe_Handle,'Hwe_SetStrictMediaEnforcment'); Assert (@Hwe_SetStrictMediaEnforcment <> Nil);
      @Hwe_IsHookProgramActive := GetProcAddress(Hwe_Handle,'Hwe_IsHookProgramActive'); Assert (@Hwe_IsHookProgramActive <> Nil);
      @Hwe_GetLogDiscDetectionStatus := GetProcAddress(Hwe_Handle,'Hwe_GetLogDiscDetectionStatus'); {Assert (@Hwe_GetLogDiscDetectionStatus <> Nil);}
      @Hwe_SetLogDiscDetectionStatus := GetProcAddress(Hwe_Handle,'Hwe_SetLogDiscDetectionStatus'); {Assert (@Hwe_SetLogDiscDetectionStatus <> Nil);}
      @Hwe_GetSDKVersionInformation := GetProcAddress(Hwe_Handle,'Hwe_GetSDKVersionInformation');
      @Hwe_GetBlankAllowed := GetProcAddress(Hwe_Handle,'Hwe_GetBlankAllowed');
      @Hwe_StartBinaryWriting := GetProcAddress(Hwe_Handle,'Hwe_StartBinaryWriting');
      @Hwe_GetProgressInterface := GetProcAddress(Hwe_Handle,'Hwe_GetProgressInterface');
      except begin
        FreeLibrary(Hwe_Handle);
        Hwe_Handle := 0;
        result := False;
      end;
    end;
  except begin
    HweLoadCount := 0;
    result := false;
  end;
  end;
end;

function HweLoaded: Boolean;
begin
  Result := Hwe_Handle > 32;
end;

procedure FreeHwe;
begin
  if not HweLoaded then Exit;
  Dec(HweLoadCount);
  if HweLoadCount <= 0 then
  begin
    @Hwe_Init := Nil;
    @Hwe_Free := Nil;
    @Hwe_GetDvdWritersCount := Nil;
    @Hwe_GetDvdReadersCount := Nil;
    @Hwe_GetDvdWriterName := Nil;
    @Hwe_SetWriter := Nil;
    @Hwe_GetLayerStatus := Nil;
    @Hwe_GetMediaStatus := Nil;
    @Hwe_GetWriteStatus := Nil;
    @Hwe_GetMediaType := Nil;
    @Hwe_StartWriting := Nil;
    @Hwe_StartDataWriting := Nil;
    @Hwe_StartIsoWriting := Nil;
    @Hwe_Blank := Nil;
    @Hwe_RegisterCallback := Nil;
    @Hwe_AbortWrite := Nil;
    @Hwe_CloseTray := Nil;
    @Hwe_OpenTray := Nil;
    @Hwe_EmptyMediaSize := Nil;
    @Hwe_SendKey := Nil;
    @Hwe_GetDriveStatus := Nil;
    @Hwe_GetDeviceName := Nil;
    @Hwe_SetWriterBydosDevice := Nil;
    @Hwe_GetWriterDosDevice := Nil;
    @Hwe_GetMediaVolumeName := Nil;
    @Hwe_LockTray := Nil;
    @Hwe_UnlockTray := Nil;
    @Hwe_SetMediaToRom := Nil;
    @Hwe_GetWriterMode := Nil;
    @Hwe_SetWriterMode := Nil;
    @Hwe_SetLogFile := Nil;
    @Hwe_GetWriteSpeeds := Nil;
    @Hwe_SetWriteSpeed := Nil;
    @Hwe_GetMediaWriteSpeeds := Nil;
    @Hwe_GetEnablePolling := Nil;
    @Hwe_SetEnablePolling := Nil;
    @Hwe_GetSendWebStats := Nil;
    @Hwe_SetSendWebStats := Nil;
    @Hwe_GetDontAutoEject := Nil;
    @Hwe_SetDontAutoEject := Nil;
//    @Hwe_StartStreamWriting := Nil;
    @Hwe_GetWriterSupportedMediaList := Nil;
    @Hwe_IsDvdWriter := Nil;
    @Hwe_SetAnyDVDActive := nil;
    @Hwe_GetPreferDVDRSao := Nil;
    @Hwe_SetPreferDVDRSao := Nil;
    @Hwe_GetSuitableMediaList := Nil;
    @Hwe_DmaEnabled := Nil;
    @Hwe_SetLayerBreak := Nil;
    @Hwe_GetLayerBreak := Nil;
    @Hwe_GetCheckAndFixIfo := Nil;
    @Hwe_SetCheckAndFixIfo := Nil;
    @Hwe_SetIsoPath := Nil;
    @Hwe_GetIsoPath := Nil;
    @Hwe_GetStrictMediaEnforcment := Nil;
    @Hwe_SetStrictMediaEnforcment := Nil;
    @Hwe_IsHookProgramActive := Nil;
    @Hwe_GetSDKVersionInformation := Nil;
    @Hwe_GetBlankAllowed := nil;
    @Hwe_StartBinaryWriting := nil;
    @Hwe_GetProgressInterface := nil;

    FreeLibrary(Hwe_Handle);
    Hwe_Handle := 0;
    HweLoadCount := 0;
  end;
end;



function GetNumberOfSectorsOfEmptyMedia(aProfile: Word): Int64;
begin
  case aProfile of
   MEDIATYPE_UNKNOWN    : Result := 0;       // No MEDIATYPE
   MEDIATYPE_NRDISC     : Result := 0;    // Non removable disc
   MEDIATYPE_RMDISC     : Result := 0;   // Removable Disc
   MEDIATYPE_MOE        : Result := 0;    // Magneto Optical Erasable
   MEDIATYPE_OPTICWO    : Result := 0;  // Optical Write Once
   MEDIATYPE_ASMO       : Result := 0;   // Advanced Storage - Magneto Optical
   MEDIATYPE_CDROM      : Result := 0;     // CD-ROM (not writable)
   MEDIATYPE_CDR        : Result := 445500;    // CD-R (Write once) - 99 minutes
   MEDIATYPE_CDRW       : Result := 360000;   // CD-RW (R/W) - 80 minutes
   MEDIATYPE_DVDROM     : Result := 0;    // DVD-ROM (not writable)
   MEDIATYPE_DVDR       : Result := 2297888;  // DVD-R
   MEDIATYPE_DVDRAM     : Result := 2297888; // DVD-RAM
   MEDIATYPE_DVDRW      : Result := 2297888; // DVD-RW Restricted overwrite
   MEDIATYPE_DVDRWSEQ   : Result := 2297888;// DVD-RW Sequential Recording
   MEDIATYPE_DLDVDRJMP  : Result := 4590208; // Dual Layer DVD-R
   MEDIATYPE_DLDVDR     : Result := 4171104; // Dual Layer DVD-R
   MEDIATYPE_DVDPLUSRW  : Result := 2295104; // DVD+RW           // 714544 for 80 mm disc
   MEDIATYPE_DVDPLUSR   : Result := 2295104;  // DVD+R
   MEDIATYPE_DDCDROM    : Result := 0;   // Double Density CD-ROM
   MEDIATYPE_DDCDR      : Result := 720000;  // Double Density CD-R (Write once)
   MEDIATYPE_DDCDRW     : Result := 720000; // Double Density CD-RW
   MEDIATYPE_DLDVDPLUSRW: Result := 4590208; // Dual Layer DVD+RW
   MEDIATYPE_DLDVDPLUSR : Result := 4590208; // Dual Layer DVD+R
   MEDIATYPE_HDBURNCDROM: Result := 0;
   MEDIATYPE_HDBURNCDR  : Result := 720000; // HD-BURN CD-R
   MEDIATYPE_HDBURNCDRW : Result := 720000; // HD-BURN CD-R
   MEDIATYPE_BDROM        : Result := 0;        //
   MEDIATYPE_BDRS         : Result := 12219392; // Blu ray R
   MEDIATYPE_BDRR         : Result := 12219392; // Blu ray R
   MEDIATYPE_BDRE         : Result := 12219392; // Blu Ray RW

   else result := 0;
  End;
end;

function GetMediaName(aMediaType: Word): String;
begin
  case aMediaType of
   MEDIATYPE_UNKNOWN    : Result := '';       // No profile
   MEDIATYPE_NRDISC     : Result := 'Hard Disc';    // Non removable disc
   MEDIATYPE_RMDISC     : Result := 'Removable Disc';   // Removable Disc
   MEDIATYPE_MOE        : Result := 'MO Erasable';    // Magneto Optical Erasable
   MEDIATYPE_OPTICWO    : Result := 'Optical WORM';  // Optical Write Once
   MEDIATYPE_ASMO       : Result := 'AS-MO';   // Advanced Storage - Magneto Optical
   MEDIATYPE_CDROM      : Result := 'CD-ROM';     // CD-ROM (not writable)
   MEDIATYPE_CDR        : Result := 'CD-R';    // CD-R (Write once)
   MEDIATYPE_CDRW       : Result := 'CD-RW';   // CD-RW (R/W)
   MEDIATYPE_DVDROM     : Result := 'DVD-ROM';    // DVD-ROM (not writable)
   MEDIATYPE_DVDR       : Result := 'DVD-R';  // DVD-R
   MEDIATYPE_DVDRAM     : Result := 'DVD-RAM'; // DVD-RAM
   MEDIATYPE_DVDRW      : Result := 'DVD-RW'; // DVD-RW Restricted overwrite
   MEDIATYPE_DVDRWSEQ   : Result := 'DVD-RW Sequential';// DVD-RW Sequential Recording
   MEDIATYPE_DLDVDR     : Result := 'Dual Layer DVD-R Sequential';
   MEDIATYPE_DLDVDRJMP  : Result := 'Dual Layer DVD-R Jump recording';
   MEDIATYPE_DVDPLUSRW  : Result := 'DVD+RW'; // DVD+RW
   MEDIATYPE_DVDPLUSR   : Result := 'DVD+R';  // DVD+R
   MEDIATYPE_DDCDROM    : Result := 'DD CD-ROM';   // Double Density CD-ROM
   MEDIATYPE_DDCDR      : Result := 'DD CD-R';  // Double Density CD-R (Write once)
   MEDIATYPE_DDCDRW     : Result := 'DD CD-RW'; // Double Density CD-RW
   MEDIATYPE_DLDVDPLUSRW: Result := 'Dual Layer DVD+RW'; // Dual Layer DVD+RW
   MEDIATYPE_DLDVDPLUSR : Result := 'Dual Layer DVD+R'; // Dual Layer DVD+R
   MEDIATYPE_HDBURNCDROM: Result := 'HD-BURN CD-ROM';
   MEDIATYPE_HDBURNCDR  : Result := 'HD-BURN CD-R'; // HD-BURN CD-R
   MEDIATYPE_HDBURNCDRW : Result := 'HD-BURN CD-RW'; // HD-BURN CD-R
   MEDIATYPE_BDROM        : Result := 'BD-ROM';                  // BluRay Disc ROM
   MEDIATYPE_BDRS         : Result := 'BD-R Sequential';         // BluRay Disc Sequential Recording
   MEDIATYPE_BDRR         : Result := 'BD-R Random';             // BluRay Disc Random Recording
   MEDIATYPE_BDRE         : Result := 'BD-RE';                   // BluRay Disc Erasable
   MEDIATYPE_HD_DVDROM    : Result := 'HD DVD-ROM';              // HD DVD (not writable)
   MEDIATYPE_HD_DVDR      : Result := 'HD DVD-R';                // HD DVD-R
   MEDIATYPE_HD_DVDRAM    : Result := 'HD DVD-RAM';              // HD DVD-RAM
   else result := '';
  End;
end;

function IntToMo (Value : Int64) : string;
    // Returns a value into a formatted string in O, Ko, Mo or Go
var TmpVal : Double;

begin
  if Value < 1024 then
  begin
    Result := format('%-4u %s',[Value,STR_Bytes]);
  end else
  begin
    TmpVal := Value;
    TmpVal := TmpVal / 1024;
    if TmpVal < 1024 then
    begin
      if TmpVal < 1000 then
        Result := Format ('%.3g %s',[TmpVal,STR_KBytes])
      else
        Result := Format ('%u %s',[trunc(TmpVal),STR_KBytes]);
    end else
    begin
      TmpVal := TmpVal / 1024;
      if TmpVal < 1024 then
      begin
        if TmpVal < 1000 then
          Result := Format ('%.3g %s',[TmpVal,STR_MBytes])
        else
          Result := Format ('%u %s',[trunc(TmpVal),STR_MBytes]);
      end else
      begin
        TmpVal := TmpVal / 1024;
        if TmpVal < 1024 then
        begin
          if TmpVal < 1000 then
            Result := Format ('%.3g %s',[TmpVal,STR_GBYTES])
          else
            Result := Format ('%u %s',[trunc(TmpVal),STR_GBYTES]);
        end else
        begin
          TmpVal := TmpVal / 1024;
          if TmpVal < 1000 then
            Result := Format ('%.3g %s',[TmpVal,STR_TBYTES])
          else
            Result := Format ('%u %s',[trunc(TmpVal),STR_GBYTES]);
          end;
      end;
    end;
  end;
end;




{ TAbstractHeadlessEngine }

procedure CbProc(Hwe: Longword; WriteOp: Longword; Progress: NativeUInt; UserParam: NativeUInt); StdCall;
begin
  Try
    TBurningEngine(UserParam).DoNotify(WriteOp,Progress);
    except ;
  End;
end;

{ TBurningEngine }

procedure TBurningEngine.AbortQuery;
begin
  if FHWe_Available and assigned(Hwe_AbortWrite) then
    Hwe_AbortWrite(Hwe);
end;

// HERE
procedure TBurningEngine.GetWriteSpeeds(S: TStrings);
var
  SpeedList: Array [0..49] of integer;
  EntryCount: Integer;
  i: integer;
begin
  if assigned(S) and FHWe_Available and assigned(Hwe_GetWriteSpeeds) then
  begin
    Hwe_GetWriteSpeeds(Hwe,@SpeedList,50);

    EntryCount := 50;
    S.BeginUpdate;
    S.clear;
    i := 0;
    while (i < EntryCount) and (SpeedList[i] > 0) do
    begin
      S.AddObject(inttostr(SpeedList[i]),TObject(SpeedList[i]));
      inc(i);
    end;
    S.EndUpdate;
  end;
end;

procedure TBurningEngine.GetMediaWriteSpeeds(S: TStrings);
var
  SpeedList: Array [0..49] of Single;
  BpsList: Array [0..49] of integer;
  EntryCount: Integer;
  i: integer;
  str: string;
  {$if CompilerVersion > 21.0}  // Delphi XE
  Fs: TFormatSettings;
  {$else}
    oldDC: char;
  {$ifend}

begin
  EntryCount := 50;
  if assigned(S) and FHWe_Available and Assigned(Hwe_GetMediaWriteSpeeds) and Assigned(Hwe_GetWriteSpeeds) then
  begin
    Hwe_GetMediaWriteSpeeds(Hwe,@SpeedList,EntryCount);
    Hwe_GetWriteSpeeds(Hwe,@BpsList,EntryCount);

    S.BeginUpdate;
    S.clear;
    {$if CompilerVersion > 21.0}  // Delphi XE
    Fs := FormatSettings;
    Fs.DecimalSeparator := '.';
    i := 0;
    while (i < EntryCount) and (SpeedList[i] > 0) do
    begin
      str := Format('x%f',[SpeedList[i]], Fs);
      while (str[length(str)] = '0') do
        str := Copy(str,1,length(str)-1);
      if (str[length(str)] = ',') or (str[length(str)] = '.') then
        str := Copy(str,1,length(str)-1);

      S.AddObject(str,TObject(BpsList[i]));
      inc(i);
    end;
    {$else}
    oldDC := decimalSeparator;
    decimalSeparator := '.';
    i := 0;
    while (i < EntryCount) and (SpeedList[i] > 0) do
    begin
      str := Format('x%f',[SpeedList[i]]);
      while (str[length(str)] = '0') do
        str := Copy(str,1,length(str)-1);
      if (str[length(str)] = ',') or (str[length(str)] = '.') then
        str := Copy(str,1,length(str)-1);

      S.AddObject(str,TObject(BpsList[i]));
      inc(i);
    end;
    decimalSeparator := oldDC;
    {$ifend}
    S.EndUpdate;
  end;
end;

procedure TBurningEngine.SetWriteSpeed(S: TStrings;Index: Integer);
begin
  if FHWe_Available and Assigned(Hwe_SetWriteSpeed) and (S.Count > Index) then
    Hwe_SetWriteSpeed(Hwe,integer(S.Objects[Index]));
end;

procedure TBurningEngine.AsyncDataStart(Folder, VolumeName: String);
Var
  F, V: RawByteString;
begin
  FBurnMode := BM_DATA;
  F := 'UTF8:'+UTF8Encode(Folder);
  V := 'UTF8:'+UTF8Encode(VolumeName);
  if FHWe_Available and Assigned(Hwe_StartDataWriting) then
    Hwe_StartDataWriting(Hwe,pAnsiChar(F),pAnsiChar(V));
end;

procedure TBurningEngine.AsyncISOStart(IsoPath: String);
Var
  F: RawByteString;
begin
  FBurnMode := BM_ISOIMAGE;
  F := 'UTF8:'+UTF8Encode(IsoPath);
  if FHWe_Available and Assigned(Hwe_StartIsoWriting) then
    Hwe_StartIsoWriting(Hwe,pansichar(F));
end;

procedure TBurningEngine.AsyncBdvStart(Folder, VolumeName: String);
Var
  F, V: RawByteString;
begin
  FBurnMode := BM_BLURAYVIDEO;
  F := 'UTF8:'+UTF8Encode(Folder);
  V := 'UTF8:'+UTF8Encode(VolumeName);
  if FHWe_Available and Assigned(Hwe_StartBdvWriting) then
    Hwe_StartBdvWriting(Hwe,pAnsiChar(F),pAnsiChar(V));
end;

procedure TBurningEngine.AsyncBinaryStart(
  aDataStream: IVSOBurningAPIDataReader);
begin
  FBurnMode := BM_STREAM;
  if FHWe_Available and Assigned(Hwe_StartBinaryWriting) then
    Hwe_StartBinaryWriting(Hwe,aDataStream);
end;

procedure TBurningEngine.AsyncBlank(EjectOnSuccess: boolean; EjectOnFail: Boolean);
begin
  if FHWe_Available and Assigned(Hwe_Blank) then
  begin
    FBurnMode := BM_ERASE;
    Hwe_Blank(Hwe,EjectOnSuccess,EjectOnFail);
  end;
end;

procedure TBurningEngine.AsyncStart(Folder, VolumeName: String);
Var
  F, V: RawByteString;
begin
  FBurnMode := BM_DVDVIDEO;
  F := 'UTF8:'+UTF8Encode(Folder);
  V := 'UTF8:'+UTF8Encode(VolumeName);
  if FHWe_Available and Assigned(Hwe_StartWriting) then
    Hwe_StartWriting(Hwe,pAnsiChar(F),pAnsiChar(V));
end;

function TBurningEngine.CloseTray: Boolean;
begin
  if FHWe_Available and Assigned(Hwe_CloseTray) then
    result := Hwe_CloseTray(Hwe)
  else result := False;
end;

constructor TBurningEngine.Create(const Path: String;
  const WndH: THandle);
var
  P: RawByteString;

begin
  P := 'UTF8:' + UTF8Encode(Path);
  FHWe_Available := LoadHwe;
  FHWe_Available := FHWe_Available and (Hwe_Init(pAnsiChar(P),WndH,Hwe) in [HWE_OK,HWE_NO_DVD_WRITERS]);
  if FHWe_Available and assigned(Hwe_RegisterCallback) then
    Hwe_RegisterCallback(Hwe,@CbProc,NativeUint(Self));
end;

destructor TBurningEngine.destroy;
begin
//  if assigned(Fisc) then FreeMem(Fisc);
  try
    if assigned(Hwe_Free) and (Hwe <> 0) then
      Hwe_Free(Hwe);
    Hwe := 0;
    FreeHwe;
    except
  end;
  inherited;
end;

function TBurningEngine.DeviceName(C: AnsiChar): string;
var
  Buff: TBytes;
begin
  if FHWe_Available and assigned(Hwe_GetDeviceName) then
  begin
    setlength(buff,4096);
    if Hwe_GetDeviceName(Hwe,C,PAnsiChar(Buff),length(Buff)) then
    begin
        result := TEncoding.UTF8.GetString(buff,0,{$IFDEF NEED_DISAMBIGUATION}System.AnsiStrings.{$ENDIF}strlen(PAnsiChar(Buff)));
    end else
        result := '';
  end;
end;

procedure TBurningEngine.DoNotify(WriteOp: Longword; Progress: NativeUInt);
var
  S: String;
  Sev: TLog_Kind;
  StrP: pAnsiChar;
begin
  FCancelAllowed := False;
  case TWriteOperation(WriteOp) of
    WRO_WRITING: begin
      FCancelAllowed := True;
    end;
    WRO_DISC_ACCEPTED: begin
        case MediaStatus of
          HWE_READY_TO_WRITE: begin
            FWriteAllowed := True;
          end;
          HWE_FULL_RW: begin
            FWriteAllowed := True;
          end
          else begin
            FWriteAllowed := False;
          end;
        end;
    end;
    WRO_DISC_REJECTED: begin
      FWriteAllowed := False;
    end;
  End;

  if (integer(WriteOp) = Ord(WRO_LOG)) then
  begin
    if assigned(FOnLog) then
    begin
      StrP := PAnsiChar(Progress);
      S := string(StrP);
      if pos('Error: ',S) = 1 then
      begin
        Sev := LK_ERROR;
        delete(S,1,6);
      end else
      if pos('Warning: ',S) = 1 then
      begin
        Sev := LK_WARNING;
        delete(S,1,8);
      end else
      if pos('Info: ',S) = 1 then
      begin
        Sev := LK_INFO;
        delete(S,1,5);
      end else
      if pos('Critical error: ',S) = 1 then
      begin
        Sev := LK_CRITICAL;
        delete(S,1,9);
      end else begin
        Sev := LK_INFO;
      end;
      FOnLog(self,Sev,S,[]);
    End else if assigned(FOnnotify) then
    begin
      FOnNotify(self,WriteOp,Progress);
    end;
  end else begin
    if assigned(FOnnotify) then
    begin
      FOnNotify(self,WriteOp,Progress);
    end;
  end;
end;

function TBurningEngine.GetBlankAllowed: Boolean;
begin
  result := Hwe_GetBlankAllowed(Hwe);
end;

function TBurningEngine.GetBurnMode: THweBurnMode;
begin
  result := FBurnMode;
end;

function TBurningEngine.GetCheckAndFixIfo: boolean;
begin
  result := Hwe_GetCheckAndFixIfo(Hwe);
end;

procedure TBurningEngine.GetDeviceList(S: TStrings);
var
    Dl: AnsiChar;
    Dn: String;
begin
    if not assigned(S) then Exit;
    S.BeginUpdate;
    S.clear;
    for Dl := 'A' to 'Z' do
    begin
      Dn := DeviceName(Dl);
      if Dn <> '' then
        S.AddObject(Dn,pointer(Dl));
    end;
    S.EndUpdate;
end;

function TBurningEngine.GetDll_Details: String;
var
  SDKinfos: UTF8String;
  MaxLen: integer;
begin
  result := '';
  if assigned(Hwe_GetSDKVersionInformation) then
  begin
    MaxLen := 1024;
    setlength(SDKinfos,MaxLen);
    if Hwe_GetSDKVersionInformation(Hwe,MarshaledAString(SDKinfos),@MaxLen) then
    begin
      SetLength(SDKinfos,MaxLen);
      result := UTF8ToString(SDKinfos);
    end;
  end;
end;

function TBurningEngine.GetDriveStatus(DriveLetter: AnsiChar): HweDriveStatus;
begin
  if FHWe_Available and assigned(Hwe_GetDriveStatus) then
    result := HweDriveStatus(Hwe_GetDriveStatus(Hwe,DriveLetter))
  else result := DRIVESTATUS_UNKNOWN;
end;

function TBurningEngine.GetDVDRWritingModeAffinity: Longword;
begin
  Result := Hwe_GetDVDRWritingModeAffinity(Hwe);
end;

function TBurningEngine.GetLayerStatus: Integer;
begin
  if FHWe_Available and assigned(Hwe_GetLayerStatus) then
    Result := Hwe_GetLayerStatus(Hwe)
  else result := -1;
end;

function TBurningEngine.GetLogDiscDetectionStatus: boolean;
begin
  if FHWe_Available and assigned(Hwe_GetLogDiscDetectionStatus) then
    result := Hwe_GetLogDiscDetectionStatus(Hwe)
  else
    Result := True;
end;

function TBurningEngine.GetMediaStatus: Integer;
begin
  if FHWe_Available and assigned(Hwe_GetMediaStatus) then
    Result := Hwe_GetMediaStatus(Hwe)
  else result := -1;
end;

function TBurningEngine.GetOnNotify: THWENotifyEvent;
begin
  Result := FOnNotify;
end;

function TBurningEngine.GetWriterCount: integer;
begin
  if FHWe_Available and assigned(Hwe_GetDvdWritersCount) then
    Result := Hwe_GetDvdWritersCount(Hwe)
  else result := -1;
end;

function TBurningEngine.GetReaderCount: integer;
begin
  if FHWe_Available and assigned(Hwe_GetDvdReadersCount) then
    Result := Hwe_GetDvdReadersCount(Hwe)
  else result := -1;
end;

function TBurningEngine.GetWriterDosDevice(dwIndex: Integer): AnsiChar;
begin
  if FHWe_Available and assigned(Hwe_GetWriterDosDevice) then
    result := Hwe_GetWriterDosDevice(Hwe,dwIndex)
  else result := #0;
end;

procedure TBurningEngine.GetWriterList(S: TStrings);
var
  I: Integer;

begin
  if not assigned(S) then Exit;
  S.BeginUpdate;
  S.clear;
  For I := 0 to WriterCount - 1 do
  begin
    S.AddObject(writername[I],pointer(GetWriterDosDevice(I)));
  end;
  S.EndUpdate;
end;

function TBurningEngine.GetWriterMode(PlusRw: Boolean): Byte;
begin
  if FHWe_Available and assigned(Hwe_GetWriterMode) then
    Result := Hwe_GetWriterMode(Hwe,PlusRw)
  else result := 0;
end;

function TBurningEngine.GetWriterName(Index: Integer): String;
var
  Buf: TBytes;

begin
  if FHWe_Available and assigned(Hwe_GetDvdWriterName) then
  begin
    SetLength(Buf,256);
    Hwe_GetDvdWriterName(Hwe,Index,PansiChar(Buf),Length(Buf));
    Result := TEncoding.UTF8.GetString(Buf,0,{$IFDEF NEED_DISAMBIGUATION}System.AnsiStrings.{$ENDIF}StrLen(PAnsiChar(Buf)));
  end else result := '';
end;

function TBurningEngine.GetWriteStatus: Integer;
begin
  if FHWe_Available and assigned(Hwe_GetWriteStatus) then
    Result := Hwe_GetWriteStatus(Hwe)
  else Result := -1;
end;

function TBurningEngine.GetMediaType: Integer;
begin
  if FHWe_Available and assigned(Hwe_GetMediaType) then
    Result := Hwe_GetMediaType(Hwe)
  else
    Result := -1;
end;

function TBurningEngine.LockTray: Boolean;
begin
  if FHWe_Available and assigned(Hwe_LockTray) then
    result := Hwe_LockTray(Hwe)
  else result := False;
end;

function TBurningEngine.MediaVolumeName: string;
var
  Buf: TBytes;
begin
  if FHWe_Available and assigned(Hwe_GetMediaVolumeName) then
  begin
    SetLength(Buf,256);
    result := TEncoding.UTF8.GetString(buf,0,Hwe_GetMediaVolumeName(Hwe,@buf[0],Length(Buf)));
  end else result := '';
end;

function TBurningEngine.EmptyMediaSize: longword;
begin
  if FHWe_Available and assigned(Hwe_EmptyMediaSize) then
    result := Hwe_EmptyMediaSize(Hwe)
  else result := 0;
end;

function TBurningEngine.OpenTray: Boolean;
begin
  if FHWe_Available and assigned(Hwe_OpenTray) then
    Result := Hwe_OpenTray(Hwe)
  else result := false;
end;

function TBurningEngine.SendKey(aKey: Pointer; Len: Integer): boolean;
begin
  if FHWe_Available and assigned(Hwe_SendKey) then
    Result := Hwe_SendKey(Hwe,aKey,Len)
  else result := False;
end;

function TBurningEngine.SetMediaToRom(ToRom: Boolean): Boolean;
begin
  if FHWe_Available and assigned(Hwe_SetMediaToRom) then
    Result := Hwe_SetMediaToRom(Hwe,ToRom)
  else result := False;
end;

procedure TBurningEngine.SetOnLog(const Value: TLogAddFormatedEvent);
begin
  FOnLog := Value;
  if FHWe_Available and assigned(Hwe_RegisterCallback) then
    Hwe_RegisterCallback(Hwe,@CbProc,NativeUInt(Self));
end;

procedure TBurningEngine.SetOnNotify(const Value: THWENotifyEvent);
begin
  FOnNotify := Value;
end;

procedure TBurningEngine.SetWriter(Index: Integer);
begin
  if FHWe_Available and assigned(Hwe_SetWriter) then
    Hwe_SetWriter(Hwe,Index);
end;

function TBurningEngine.SetWriterByDosDevice(C: AnsiChar): boolean;
begin
  if FHWe_Available and assigned(Hwe_SetWriterBydosDevice) then
    result := Hwe_SetWriterBydosDevice(Hwe,C)
  else result := False;
end;

function TBurningEngine.SetWriterMode(PlusRw, ToRom: Boolean): Boolean;
begin
  if FHWe_Available and assigned(Hwe_SetWriterMode) then
    Result := Hwe_SetWriterMode(Hwe,PlusRw,ToRom)
  else result := False;
end;

function TBurningEngine.UnlockTray: Boolean;
begin
  if FHWe_Available and assigned(Hwe_UnlockTray) then
    Result := Hwe_UnlockTray(Hwe)
  else
    result := False;
end;

function TBurningEngine.GetEnablePolling: Boolean;
begin
  if FHWe_Available and assigned(Hwe_GetEnablePolling) then
     Result := Hwe_GetEnablePolling(Hwe)
  else result := False;
end;

function TBurningEngine.GetHookProgramActive: boolean;
begin
  if FHWe_Available and assigned(Hwe_IsHookProgramActive) then
    result := Hwe_IsHookProgramActive(Hwe)
  else result := false;

end;

function TBurningEngine.getIsoPath: String;
var
  aPath: AnsiString;

begin
  if FHWe_Available and assigned(Hwe_GetIsoPath) then
  begin
    SetLength(aPath,32768);
    SetLength(aPath,Hwe_GetIsoPath(Hwe, @aPath[1], 32768) - 1);
    if copy(aPath,1,5) = 'UTF8:' then
      result := UTF8ToString(Copy(aPath,6,32764))
    else result := string(aPath);
  end else begin
    result := '';
  end;
end;

procedure TBurningEngine.SetEnablePolling(const Value: Boolean);
begin
  if FHWe_Available and assigned(Hwe_SetEnablePolling) then
    Hwe_SetEnablePolling(Hwe,Value);
end;

procedure TBurningEngine.SetIsoPath(const Value: String);
var
  aPath: RawByteString;

begin
  if FHWe_Available and assigned(Hwe_SetIsoPath) then
  begin
    aPath := 'UTF8:' + UTF8Encode(Value);
    Hwe_SetIsoPath(Hwe,pAnsiChar(aPath));
  end;
end;

procedure TBurningEngine.SetLogDiscDetectionStatus(const Value: boolean);
begin
  if FHWe_Available and assigned(Hwe_SetLogDiscDetectionStatus) then
    Hwe_SetLogDiscDetectionStatus(Hwe,Value);
end;

procedure TBurningEngine.SetLogfile(LFN: String);
var
  LfnP: RawByteString;

begin
  if FHWe_Available and assigned(Hwe_SetLogFile) then
  begin
    LfnP := 'UTF8:' + UTF8Encode(Lfn);
    Hwe_SetLogFile(Hwe,PAnsiChar(LFnP));
  end;
end;

procedure InternalRead(opaque: pointer;
                       Data: Pointer;
                       Count: Integer;
                       SectorIndex: Integer;
                       Var Final: boolean); stdcall;
begin
end;

//function TBurningEngine.AsyncStreamStart(AReadFunc: Pointer; Size: Integer; Stats: Boolean): PHweInputStreamContext;
//begin
//  if FHWe_Available and assigned(Hwe_StartStreamWriting) then
//  begin
//    FBurnMode := BM_STREAM;
//    if assigned(Fisc) then Freemem(FIsc);
//    Fisc := Nil;
//    Getmem(Fisc, sizeof(THweInputStreamContext) + (sizeof(int64) * 250));
//    Fisc.Opaque := self;
//    Fisc.SectorsToWrite := Size;
//    if not assigned(aReadFunc) then
//      Fisc.ReadFunc := InternalRead
//    else
//      Fisc.ReadFunc := aReadfunc;
//    Fisc.LogWriteTiming := Stats;
//    InitializeCriticalSection(Fisc.RTL_Cs);
//    Fisc.BlockCount := 250;
//    Fisc.BlockSize := 0;
//    fisc.CurrentBlock := -1;
//    Hwe_StartStreamWriting(Hwe,Fisc);
//    Result := Fisc;
//  end else result := Nil;
//end;

function TBurningEngine.GetSDKVersionInformation(out infos: string): boolean;
var
  UTFInfos: UTF8String;
  len: cardinal;

begin
  if FHWe_Available and assigned(Hwe_GetSDKVersionInformation) then
  begin
    len := 2048;
    SetLength(UTFInfos,len);
    result := Hwe_GetSDKVersionInformation(Hwe,MarshaledAString(UTFInfos),@len);
    if result then
    begin
      SetLength(UTFInfos,len);
      infos := UTF8ToString(UTfInfos);
    end;
  end else result := False;
end;

function TBurningEngine.GetSendWebStats: boolean;
begin
  if FHWe_Available and assigned(Hwe_GetSendWebStats) then
    result := Hwe_GetSendWebStats(Hwe)
  else result := False;
end;

function TBurningEngine.GetStrictMediaEnforcment: boolean;
begin
  if FHWe_Available and assigned(Hwe_GetStrictMediaEnforcment) then
    result := Hwe_GetStrictMediaEnforcment(Hwe)
  else
    result := False;
end;

procedure TBurningEngine.SetSendWebStats(const Value: boolean);
begin
  if FHWe_Available and assigned(Hwe_SetSendWebStats) then
    Hwe_SetSendWebStats(Hwe,Value);
end;

procedure TBurningEngine.SetStrictMediaEnforcment(const Value: boolean);
begin
  if FHWe_Available and assigned(Hwe_SetStrictMediaEnforcment) then
    Hwe_SetStrictMediaEnforcment(Hwe,Value);
end;

procedure TBurningEngine.SetUserLayerBreak(const Value: integer);
begin
  if FHWe_Available and assigned(Hwe_SetLayerBreak) then
    Hwe_SetLayerBreak(Hwe,Value);
end;

function TBurningEngine.IsDvdWriter: Boolean;
begin
  if FHWe_Available and assigned(Hwe_IsDvdWriter) then
    Result := Hwe_IsDvdWriter(Hwe)
  else result := False;
end;

function TBurningEngine.GetSupportedMediaList: string;
var
  buf: Tbytes;
begin
  if FHWe_Available and assigned(Hwe_GetWriterSupportedMediaList) then
  begin
    SetLength(buf,1024);
    result := TEncoding.UTF8.GetString(buf,0,Hwe_GetWriterSupportedMediaList(Hwe,@buf[0],length(Buf)));
  end else result := '';
end;

function TBurningEngine.GetUserLayerBreak: integer;
begin
  if FHWe_Available and assigned(Hwe_GetLayerBreak) then
    Result := Hwe_GetLayerBreak(Hwe)
  else result := -1;
end;

function TBurningEngine.GetDontAutoEject: boolean;
begin
  if FHWe_Available and assigned(Hwe_GetDontAutoEject) then
    result := Hwe_GetDontAutoEject(Hwe)
  else result := false;
end;

function TBurningEngine.SetAnyDVDActive(Value: Boolean): boolean;
begin
  if FHWe_Available and assigned(Hwe_SetAnyDVDActive) then
    result := Hwe_SetAnyDVDActive(Hwe, Value)
  else result := false;
end;

procedure TBurningEngine.SetCheckAndFixIfo(const Value: boolean);
begin
  Hwe_SetCheckAndFixIfo(Hwe,Value);
end;

//function TBurningEngine.SetDataCallback(Rcb: TDataCallback;
//  Rcbo: Pointer): boolean;
//begin
//  if FHWe_Available then
//    result := Hwe_SetDataCallback(Hwe,Rcb,Rcbo)
//  else
//    result := false;
//end;

function TBurningEngine.SetDataCallback(Rcb: TDataCallback;
  Rcbo: Pointer): boolean;
begin
  result := false;
end;

procedure TBurningEngine.SetDontAutoEject(const Value: boolean);
begin
  if FHWe_Available and assigned(Hwe_SetDontAutoEject) then
    Hwe_SetDontAutoEject(Hwe,Value);
end;

procedure TBurningEngine.SetDVDRWritingModeAffinity(const Value: Longword);
begin
  Hwe_SetDVDRWritingModeAffinity(Hwe,Value);
end;

procedure TBurningEngine.SetPreferDVDRSAO(const Value: boolean);
begin
  if FHWe_Available and assigned(Hwe_SetPreferDVDRSao) then
    Hwe_SetPreferDVDRSao(Hwe, value);
end;

function TBurningEngine.GetPreferDVDRSAO: boolean;
begin
  if FHWe_Available and assigned(Hwe_GetPreferDVDRSao) then
    Result := Hwe_GetPreferDVDRSao(Hwe)
  else result := False;

end;

function TBurningEngine.GetProgressInterface(
  out intf: IVSOBurningSDKProgress): boolean;
begin
  result := FHWe_Available and assigned(Hwe_GetProgressInterface) and Hwe_GetProgressInterface(Hwe,intf);
end;

function TBurningEngine.SuitableMediaList(SectorsToWrite: Longword): String;
var
  Buf: TBytes;
begin
  if FHWe_Available and assigned(Hwe_GetSuitableMediaList) then
  begin
    SetLength(Buf,1024);
    result := TEncoding.UTF8.GetString(buf,0,Hwe_GetSuitableMediaList(Hwe,Int64(SectorsToWrite) shl 11,@Buf[0],Length(Buf)));
  end else result := '';
end;

function TBurningEngine.DmaEnabled(C: AnsiChar): integer;
begin
  if FHWe_Available and assigned(Hwe_DmaEnabled) then
    Result := Hwe_DmaEnabled(Hwe,C)
  else result := -1;
end;

function TBurningEngine.DmaName(C: AnsiChar): string;
var
  buf: Tbytes;

begin
  if FHWe_Available and assigned(Hwe_DmaName) then
  begin
    SetLength(Buf,1024);
    result := TEncoding.UTF8.GetString(buf,0,Hwe_DmaName(Hwe,C,@Buf[0],Length(Buf)));
  end else begin
    result := '';
  end;

end;

end.
