unit debug_helper;
{
  Unit d'aide au dbogage.
  Compatibilit: toutes plateformes.
  DbgOut est un remplacement de OutputDebugString.

}

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface
procedure DbgOut(const fmt: string; const params: array of const ); overload;
procedure DbgOut(const fmt: string); overload; {$IFNDEF DEBUG} inline; {$ENDIF}

implementation

{$IFDEF DEBUG}
uses
{$IFDEF MSWINDOWS}
  Winapi.Windows,
{$ENDIF MSWINDOWS}
{$IFDEF ANDROID}
  System.Classes,
  System.Syncobjs,
  path.types,
  path.utils,
  utils.highres_time,
  Androidapi.Log,
  Androidapi.Helpers,
  Androidapi.NativeActivity,
  AndroidApi.JNI,
  AndroidApi.JNIBridge,
  AndroidApi.JNI.NET,
  AndroidApi.JNI.Media,
  AndroidApi.JNI.JavaTypes,
  Androidapi.JNI.GraphicsContentViewText,
  Posix.sysstat,
  Posix.Stdio,
{$ENDIF}
  System.SysUtils
 ;


{$IFDEF ANDROID}
var
  DbgF: string = '';
  DbgLock: TCriticalSection;
  start_time: THRChrono;

function StringToTBytes(const src: string): TBytes;
var
  len: integer;
begin
  len := length(src)*2;
  setlength(result,Len);
  move(pchar(src)^,result[0],len);
end;

procedure DroidShowFile;
var
  JF: JFile;
  Uri: Jnet_Uri;
  S: JString;
  Intent: JIntent;
  Pa: TJavaObjectArray<JString>;

begin
  Pa := TJavaObjectArray<JString>.Create(1);
  Pa.Items[0] := StringToJString(TPath.ExtractDirectory(DbgF));
  TJMediaScannerConnection.JavaClass.scanFile(SharedActivityContext.getApplicationContext,Pa,nil,nil);
  Jf := TJFile.JavaClass.init(StringToJString(DbgF));
  Uri := TJnet_uri.JavaClass.fromFile(Jf);
  S := TJIntent.JavaClass.ACTION_MEDIA_SCANNER_SCAN_FILE;
  Intent := TJIntent.JavaClass.init(S,Uri);
  SharedActivityContext.sendBroadcast(Intent);
end;

function __android_log_print(Priority: android_LogPriority; const Tag, Text: MarshaledAString): Integer; cdecl;
  external AndroidLogLib name '__android_log_print';


procedure DroidDbgOut(str: string);
var
  F: TFileStream;
  pm: JPackageManager;
  pi: JPackageInfo;
  tmp: TBytes;
  First_log: boolean;

begin

  DbgLock.Enter;
  try
    First_log := DbgF = '';
    if First_log then
    begin
      pm := SharedActivityContext.getPackageManager;
      pi := pm.getPackageArchiveInfo(SharedActivityContext.getPackageCodePath,0);
      DbgF:= TPath.combine('/sdcard','Debug.'+JStringToString(pi.packageName) +'.Log');
      if FileExists(DbgF) then
      begin
        RenameFile(DbgF,Dbgf+'.pending');
      end;
      Dbgf := Dbgf+'.pending';
      if FileExists(DbgF) then
      begin
        F := TFileStream.Create(DbgF,fmOpenReadWrite);
        F.Seek(0,soEnd);
      end else begin
        if not TPath.ForceFilePath(DbgF) then Exit;
        F := TFileStream.Create(DbgF,fmCreate);
        Tmp := TEncoding.UTF8.GetPreamble;
        F.Write(Tmp,length(Tmp));
      end;
      Tmp := TEncoding.UTF8.GetBytes(format('*** Debug session of %s vers. %s, %s ***'#13#10,[JStringToString(pi.packageName), JStringToString(pi.versionName),DatetimeToStr(now)]));
      F.Write(Tmp,length(Tmp));
      start_time.restart;
    end else begin
      F := TFileStream.Create(DbgF,fmOpenReadWrite);
      F.Seek(0,soEnd);
    end;

    Tmp := TEncoding.UTF8.GetBytes(format('  %.10d - %s'#13#10,[start_time.elapsed.AsUnixTimeStamp,str])) + [0];
    F.Write(Tmp,length(Tmp)- 1);
    __android_log_print(ANDROID_LOG_DEBUG,'debug',@tmp[0]);
    F.Free;
    if First_log then DroidShowFile;
  finally DbgLock.Leave;
  end;
end;

{$ENDIF}


procedure DbgOut(const fmt: string; const params: array of const);
begin
{$IFDEF MSWINDOWS}
  OutputDebugString(pchar('vso: '+format(fmt,params)));
{$ELSE}
{$IFDEF ANDROID}
  DroidDbgOut(format(fmt,params));
{$ELSE}
  WriteLn(output,'vso: ',format(fmt,params));
{$ENDIF}
{$ENDIF}
end;

procedure DbgOut(const fmt: string);
begin
{$IFDEF MSWINDOWS}
  OutputDebugString(pchar('vso: '+fmt));
{$ELSE}
{$IFDEF ANDROID}
  DroidDbgOut(fmt);
{$ELSE}
  WriteLn(output,'vso: ',fmt);
{$ENDIF}
{$ENDIF}
end;

{$ELSE !DEBUG}
procedure DbgOut(const fmt: string; const params: array of const); overload;
begin
end;

procedure DbgOut(const fmt: string); inline;
begin
end;
{$ENDIF !DEBUG}

{$IFDEF ANDROID}
{$IFDEF DEBUG}
initialization
  DbgLock := TCriticalSection.Create;

finalization
  if DbgF <> '' then
  begin
    RenameFile(DbgF,DbgF.Substring(0,length(DbgF)-8));
    DroidShowFile;
  end;

  DbgLock.free;
{$ENDIF}
{$ENDIF}
end.
