unit localiser.interfaces;
interface
uses
System.Classes;

type
  ILocaliser = interface
    ['{FA42F529-54E0-4eeb-965A-B83D4660390F}']
    // Interface to access the localise routine from inside a DLL or another component
    function localise(original: string): string; stdcall;
  end;

  ILocaliserClient = interface
    ['{D522A5FE-F6AA-495a-8603-9968CC34CF74}']
    // Interface to access the localise routine from inside a DLL or another component
    procedure register_localiser(loc: ILocaliser);
  end;

  IDesignTimeAdditionalSourceFileParsing = interface
    ['{57B150CD-4F9E-43a5-B7B0-78D2A0CE69AA}']
    // add to the passed stringlist all the source files that the engine want to parse
    function add_source(Sl: TStringList): integer;
  end;

  ILocalisedComponent = interface
    ['{559BBBB6-8D28-47b8-BFE8-B94F802249E7}']
    function getparse_properties: boolean;
    function getparse_components: boolean;

    function getlocalised_count: integer;
    function getlocalised(index: integer): string;
    function getlocalised_propname(index: integer): string;
    procedure setlocalised(index: integer; const value: string);



    // if returning TRUE, published properties of the component will be parsed
    property parse_properties: boolean read getparse_properties;
    // if returning TRUE, child components of the component will be parsed
    property parse_components: boolean read getparse_components;
    // return the number of strings to be localised
    property localised_count: integer read getlocalised_count;

    // accessor to localised strings
    property localised[index: integer]: string read getlocalised write setlocalised;
    property localised_propname[index: integer]: string read getlocalised_propname;
  end;

  ILocalisedComponentTag = interface
    ['{499D759D-2EDE-451A-84C8-976D236C7227}']
//    function gettag(index: integer): string;
    function getcomment(index: integer): string;

    // optionnal tag for each localised properties
//    property tag[index: integer]: string read gettag;
    // optionnal comment for each localised properties
    property comment[index: integer]: string read getcomment;
  end;

  {$IFDEF NOT_LOCALISED}
  function localise(const S: string): string; inline;
  {$ENDIF}


implementation
  {$IFDEF NOT_LOCALISED}
  function localise(const S: string): string; inline;
  begin
    result := S;
  end;
  {$ENDIF}

end.
